/**HEADER********************************************************************
* 
* Copyright (c) 2010 Avnet Japan Co.,Ltd (NIKE Borad Project.)
* All Rights Reserved
*
*************************************************************************** 
*
* $FileName: sensor_task.c$
* $Version : 3.0.14.0$
* $Date    : May-26-2009$
*
* Comments:
*
*   This file contains the source for the ADC example program.
*   Two channels are running, one is running in loop, the second one
*   only once.
*
*END************************************************************************/

#include <mqx.h>
#include <bsp.h>
#include <sensor.h>
#include <Sensor_Task.h>
#include <Signal_Task.h>

#include <sensor_adc_list.h>
#include <string.h>

#include "NIKE_demo.h"
#include <psptypes.h>

//#define SENSOR_DEBUG
#define SENSOR_ENABLE_EASY_DEMO

/* Sensor Default Config Parameter */
#define SENSOR_DEFAULT_ADC_THRESHOLD_X           120
#define SENSOR_DEFAULT_ADC_THRESHOLD_Y            48
#define SENSOR_DEFAULT_ADC_THRESHOLD_Z           176

#define SENSOR_NORMAL_ADC_THRESHOLD              16

#define SENSOR_DEMO_ADC_THRESHOLD                 80

#define SENSOR_DEFAULT_ADC_THRESHOLD_Z_IMPACT    480
#define SENSOR_DEFAULT_ADC_THRESHOLD_Z_SHAKE     100

#define SENSOR_3AXIS_MOVEMENT_WAIT_CNT           5  /* for wait 3*100ms */
//#define SENSOR_DELAY_TIME                        100  /* for wait 100ms */
#define SENSOR_DELAY_TIME                        200  /* for wait 100ms */

#define SENSOR_DEFAULT_ADC_THRESHOLD_WEAK        SENSOR_DEFAULT_ADC_THRESHOLD
#define SENSOR_DEFAULT_ADC_THRESHOLD_STRONG      256
#define SENSOR_DEFAULT_ADC_THRESHOLD_IMPACT      300

#define SENSOR_3AXIS_SIZE                        3

/* Sensor local parameter */
#define SENSOR_FILE_NUM_MAX                SENSOR_ADC_LIST_CNANNEL_NUM_LIST
#define SENSOR_MEATURE_NUM                 SENSOR_FILE_NUM_MAX
#define SENSOR_DIFF_CHECK_NUM              SENSOR_ADC_LIST_DIFF_CHECK_NUM_MAX

#define SENSOR_DELAY(x)       _time_delay(x)
#define SENSOR_DELAY_500()    _time_delay(500)
#define SENSOR_INIT_DELAY()   _time_delay(1000)

#ifdef SENSOR_DEBUG                   
#define SENSOR_PRINTF(X)              \
  if(SensorEnableDebugMode != FALSE){ \
    X;                                \
  }
#else
#define SENSOR_PRINTF(X)
#endif

#ifdef SENSOR_DEBUG
#define SENSOR_DEBUG_PRINTF(X)               X
#else
#define SENSOR_DEBUG_PRINTF(X)               
#endif

#define SENSOR_3AXIS_MOVEMENT                1


/* Three axes Table in sensor task */
enum SENSOR_3AXIS_TABLE{
  SENSOR_3AXIS_X = 0,
  SENSOR_3AXIS_Y,
  SENSOR_3AXIS_Z
};

/* Sensor State Machine Status List */
enum _sensor_status_list{
  SENSOR_ST_INIT = 0,
  SENSOR_ST_NORMAL,
  SENSOR_ST_WAIT,
  SENSOR_ST_NO_CHANGE
};

/* Local Function */
_mqx_int sensorParamInit( void);
_mqx_int sensorMainOpen( void);
_mqx_int sensorAdcChannelOpen( const SENSOR_ADC_INIT_CHANNEL_STRUCT *pList, _mqx_int num);
_mqx_int sensorMainClose( char_ptr open_name_ptr);
_mqx_int sensorAllChanneClose( void);
SENSOR_PARAM_PTR sensorGetParamAddress( void);

void sensorAdcMeasure( void);
void sensorChangeModeFunc( void);

_mqx_int sensor_3axis_acceleration_change_check( void);

/* Sensor Mode Function */
_mqx_int sensor_3axis_check_normal( void *pData);
_mqx_int sensor_3axis_check_strength( void *pData);
_mqx_int sensor_3axis_check_z_shake( void *pData);

/* Sensor State Machine Control Function */
void  sensor_status_func( void);
uchar sensor_status_func_init( void);
uchar sensor_status_func_normal( void);
uchar sensor_status_func_wait( void);

/* Sensor Only Global Value */
FILE_PTR  pSensorMainADC;
FILE_PTR  pSensorAdcFile[SENSOR_FILE_NUM_MAX];

SENSOR_ADC_VALUE_TYPE SensorMeasureData[SENSOR_MEATURE_NUM];
_mqx_int SensorAdcDelta[SENSOR_MEATURE_NUM];
boolean  SensorEnableChangeMode;

#ifdef SENSOR_DEBUG
boolean  SensorEnableDebugMode;
#endif

uchar SensorWaitCnt;
uchar Sensor3axisMode;
uchar SensorStatus;
_mqx_int SensorOldEffectChange;

#ifdef SENSOR_ENABLE_EASY_DEMO
//_mqx_int SensorToggleParam;
uint_16  SensorDemoThreshold;
#endif

SENSOR_PARAM  SensorParam;

/* Sensor Default Action Parameter Table */
//const SENSOR_ACT_TYPE_PARAM SensorDefaultActParam[SENSOR_3AXIS_ACTION_TYPE_SIZE] = {
//    {SENSOR_DEFAULT_ADC_THRESHOLD_X,        SIGNAL_EFFECT_TYPE_DISTORTION, 0    },
//    {SENSOR_DEFAULT_ADC_THRESHOLD_Y,        SIGNAL_EFFECT_TYPE_ECHO,       0    },
//    {SENSOR_DEFAULT_ADC_THRESHOLD_Z,        SIGNAL_EFFECT_TYPE_NORMAL,     0    },
//    {SENSOR_DEFAULT_ADC_THRESHOLD_IMPACT,   SIGNAL_EFFECT_TYPE_FUZZ,       0    },
//    {SENSOR_DEFAULT_ADC_THRESHOLD_Z_SHAKE,  SIGNAL_EFFECT_TYPE_NORMAL,     0    }
//};

//const SENSOR_ACT_TYPE_PARAM SensorDefaultActParam[SENSOR_3AXIS_ACTION_TYPE_SIZE] = {
//    {SENSOR_DEFAULT_ADC_THRESHOLD_X,        SIGNAL_EFFECT_ID_DISTORTION_LIMIT   },
//    {SENSOR_DEFAULT_ADC_THRESHOLD_Y,        SIGNAL_EFFECT_ID_ECHO         },
//    {SENSOR_DEFAULT_ADC_THRESHOLD_Z,        SIGNAL_EFFECT_ID_NORMAL,      },
//    {SENSOR_DEFAULT_ADC_THRESHOLD_IMPACT,   SIGNAL_EFFECT_ID_FUZZ,        },
//    {SENSOR_DEFAULT_ADC_THRESHOLD_Z_SHAKE,  SIGNAL_EFFECT_ID_NORMAL,      }
//};

const SENSOR_ACT_TYPE_PARAM SensorDefaultActParam[SENSOR_3AXIS_ACTION_TYPE_SIZE] = {
    {SENSOR_DEFAULT_ADC_THRESHOLD_X,        2 },
    {SENSOR_DEFAULT_ADC_THRESHOLD_Y,        1 },
    {SENSOR_DEFAULT_ADC_THRESHOLD_Z,        0 },
    {SENSOR_DEFAULT_ADC_THRESHOLD_IMPACT,   0 },
    {SENSOR_DEFAULT_ADC_THRESHOLD_Z_SHAKE,  0 }
};

/* Sensor ADC device init struct */
const SENSOR_INIT_STRUCT Sensor_adc_init = {
    SENSOR_ADC_RESOLUTION_DEFAULT,     /* resolution */
};



/* O̕ω */
int_32	b_SensorAdcDelta[4];

/* ɐω̃`^O */
int_32	SensorAdcDelta_count[4];

/* m̐΂̑ω */
int_32	t_SensorAdcDelta[4];


/*FUNCTION*****************************************************************
* 
* Function Name    : sensorGetParamAddress
* Input Value      : None
*
* Returned Value   : SENSOR_PARAM_PTR - Sensor param pointer

* Comments         : Sensor Parameter Address
* 
*END*********************************************************************/
SENSOR_PARAM_PTR sensorGetParamAddress( void)
{
    return &SensorParam;
}


/*FUNCTION*****************************************************************
* 
* Function Name    : sensorCtrlSetDebugMode
* Argument         : boolean enable
*                      TRUE  :  Enable
*                      FALSE :  Disable
*
* Returned Value   : _mqx_int - SENSOR_OK

* Comments         : Sensor debug mode Setting
* 
*END*********************************************************************/
_mqx_int sensorCtrlSetDebugMode( boolean enable)
{
#ifdef SENSOR_DEBUG
  SensorEnableDebugMode = enable;
#endif
  return SENSOR_OK;
}


/*FUNCTION*****************************************************************
* 
* Function Name    : sensorCtrlSetAccelerationThreshold
* Input    Value   : uchar type - Action Type
*                    _mqx_int value - 
* Returned Value   : _mqx_int - SENSOR_ERR_INVALID_ARG or SENSOR_OK

* Comments         : Sensor Threshold Setting
* 
*END*********************************************************************/
_mqx_int sensorCtrlSetActionThreshold( uchar type, uint_16 value)
{
    _mqx_int return_code = SENSOR_OK;
    SENSOR_PARAM_PTR pParam = sensorGetParamAddress();

    if(type >= SENSOR_3AXIS_ACTION_TYPE_MAX || (value >= SENSOR_3AXIS_THRESHOLD_MAX) ){
        return_code = SENSOR_ERR_INVALID_ARG;
    } else {
        pParam->act_param[type].threshold = value;
#ifdef SENSOR_ENABLE_EASY_DEMO
        SensorDemoThreshold = value;
#endif
    }
    return return_code;
}

/*FUNCTION*****************************************************************
* 
* Function Name    : sensorCtrlSetSoundOfAction
* Input    Value   : uchar type - Action Type
*                    _mqx_int value - 
* Returned Value   : _mqx_int - SENSOR_ERR_INVALID_ARG or SENSOR_OK

* Comments         : The sound by the action is set. 
* 
*END*********************************************************************/
_mqx_int sensorCtrlSetSoundOfAction( uchar type, uint_16 soundType)
{
    _mqx_int return_code = SENSOR_OK;
    SENSOR_PARAM_PTR pParam = sensorGetParamAddress();

    if(type >= SENSOR_3AXIS_ACTION_TYPE_MAX){
        return_code = SENSOR_ERR_INVALID_ARG;
    } else {
        pParam->act_param[type].soundType = soundType;
    }

    return return_code;
}

/*FUNCTION*****************************************************************
* 
* Function Name    : sensorCtrlSetMode
* Input    Value   : uchar mode - 0bit: Normal or Strength
*                                 1bit: 
* Returned Value   : _mqx_int - SENSOR_OK

* Comments         : Sensor Threshold Setting
* 
*END*********************************************************************/
_mqx_int sensorCtrlSetMode( uchar mode)
{
    SENSOR_PARAM_PTR pParam = sensorGetParamAddress();

    if( (pParam->mode & 0x0F) != (mode & 0x0F)){
       pParam->mode = mode;
       SensorEnableChangeMode = TRUE;
    }

    //if(mode & SENSOR_3AXIS_MODE_IMPACT){
    //   pParam->mode |= SENSOR_3AXIS_MODE_IMPACT;
    //}

    return SENSOR_OK;
}

/*FUNCTION*****************************************************************
* 
* Function Name    : sensorCtrlGetAdc
* Input Value      : [pData]  Adc storage buffer pointer
*                  : [num]    Number in which ADC is acquired
* Returned Value   : _mqx_int - SENSOR_ERR_INVALID_ARG or SENSOR_OK

* Comments         : Sensor Get Adc Value
* 
*END*********************************************************************/
_mqx_int sensorCtrlGetAdc( SENSOR_ADC_VALUE_TYPE *pData, _mqx_int num)
{
    _mqx_int result = SENSOR_ERR_INVALID_ARG;

    if(num > 0 && num <= SENSOR_FILE_NUM_MAX && pData != NULL){
        _mqx_int i;

        for(i=0; i<num; i++){
            pData[i] = SensorMeasureData[i];
        }

        result = SENSOR_OK;
    }

    return result;
}

/*FUNCTION*****************************************************************
* 
* Function Name    : sensorCtrlGetParam
* Input Value      : [pData]  Sensor parameter buffer pointer
* Returned Value   : _mqx_int - size: sensor parameter length
*
* Comments         : Sensor Get Parameter
* 
*END*********************************************************************/
_mqx_int sensorCtrlGetParam( SENSOR_PARAM *pData)
{
    _mqx_int result = 0;
    SENSOR_PARAM_PTR pParam = sensorGetParamAddress();

    if(pData != NULL){

        memcpy(pData, pParam, sizeof(SENSOR_PARAM));

        result = sizeof(SENSOR_PARAM);
    }

    return result;
}


/*FUNCTION*****************************************************************
* 
* Function Name    : sensor_status_func
* Input Value      : None
* Returned Value   : None
*
* Comments         : Sensor StateMachine Control Function
* 
*END*********************************************************************/
void  sensor_status_func( void)
{
    uchar changeStatus = SENSOR_ST_NO_CHANGE;

    switch(SensorStatus){
    case SENSOR_ST_INIT:
      changeStatus = sensor_status_func_init();
      break;
    case SENSOR_ST_NORMAL:
      changeStatus = sensor_status_func_normal();
      break;
    case SENSOR_ST_WAIT:
      changeStatus = sensor_status_func_wait();
      break;
    default:
      break;
    }

    if(changeStatus != SENSOR_ST_NO_CHANGE){
        SensorStatus = changeStatus;
    }
    return;
}


uchar sensor_status_func_init( void)
{
    uchar changeStatus = SENSOR_ST_NO_CHANGE;
    boolean changeStatusEnable = FALSE;
    _mqx_int valuePoint=0;

    do{
      if(SensorMeasureData[valuePoint] > 0 && SensorAdcDelta[valuePoint] <= SENSOR_NORMAL_ADC_THRESHOLD){
          /* continues */
          valuePoint++;
      }else {
          break;
      }
    }while( valuePoint < SENSOR_MEATURE_NUM);

    if(valuePoint >= SENSOR_MEATURE_NUM){
      changeStatus = SENSOR_ST_NORMAL;
      SENSOR_PRINTF(printf("[Sensor] Change St %x-> %x\n", changeStatus, SensorStatus));
    }
    return changeStatus;
}

uchar sensor_status_func_normal( void)
{
    uchar changeStatus = SENSOR_ST_NO_CHANGE;
    _mqx_int result;

    result = sensor_3axis_acceleration_change_check();

    if(result == SENSOR_3AXIS_MOVEMENT){
        SensorWaitCnt = SENSOR_3AXIS_MOVEMENT_WAIT_CNT;
        changeStatus = SENSOR_ST_WAIT;
        SENSOR_PRINTF(printf("[Sensor] Change St %x-> %x\n", SensorStatus, changeStatus));
    }

    return changeStatus;
}

uchar sensor_status_func_wait( void)
{
    uchar changeStatus = SENSOR_ST_NO_CHANGE;

    if( (--SensorWaitCnt) == 0){
        changeStatus = SENSOR_ST_NORMAL;
        SENSOR_PRINTF(printf("[Sensor] Change St %x-> %x\n", SensorStatus, changeStatus));
    }
    return changeStatus;

}

void sensorChangeModeFunc( void)
{
    if(SensorEnableChangeMode != FALSE){
        sensorParamInit();
    }
    return;
}


_mqx_int sensorParamInit( void)
{
    SENSOR_PARAM_PTR pSensParam = sensorGetParamAddress();

    pSensParam->mode = SENSOR_3AXIS_DEFAULT_MODE;

    memcpy(&pSensParam->act_param[0], &SensorDefaultActParam[0], sizeof(SensorDefaultActParam));

    _mem_zero(SensorMeasureData, sizeof(SensorMeasureData));

	b_SensorAdcDelta[0] = 0;
	b_SensorAdcDelta[1] = 0;
	b_SensorAdcDelta[2] = 0;
	b_SensorAdcDelta[3] = 0;

	/* ɐω̃`^O  */
	SensorAdcDelta_count[0] = 0;
	SensorAdcDelta_count[1] = 0;
	SensorAdcDelta_count[2] = 0;
	SensorAdcDelta_count[3] = 0;

	/* m̐΂̑ω */
	t_SensorAdcDelta[0] = 0;
	t_SensorAdcDelta[1] = 0;
	t_SensorAdcDelta[2] = 0;
	t_SensorAdcDelta[3] = 0;

    return SENSOR_OK;
}


/*FUNCTION*****************************************************************
* 
* Function Name    : sensorMainOpen
* Returned Value   : _mqx_int - IO_ERROR or IO_OK

* Comments         : Opens and initializes Sensor Main(ADC) driver.
* 
*END*********************************************************************/
_mqx_int sensorMainOpen( void )
{
  _mqx_int result = IO_ERROR;

  pSensorMainADC = fopen("sensor:", (const char*)&Sensor_adc_init);

  if(pSensorMainADC != NULL){
    result = IO_OK;
  }

  return result;

}

/*FUNCTION*****************************************************************
* 
* Function Name    : sensorAdcChannelOpen
* Returned Value   : _mqx_int - IO_ERROR or IO_OK

* Comments         : Opens and initializes Sensor Temperature(ADC) driver.
* 
*END*********************************************************************/
_mqx_int sensorAdcChannelOpen( const SENSOR_ADC_INIT_CHANNEL_STRUCT *pList, _mqx_int num)
{
    _mqx_int i=0;
    _mqx_int return_code = IO_ERROR;

    if(num > 0 && pList != NULL && num <= SENSOR_FILE_NUM_MAX){

        printf("\n");

        /* Sensor Channel Open */
        for(i=0; i< num; i++ ){
            static char dev_name[10];

            printf("Opening channel #%x ...", i+1);

            sprintf(dev_name, "sensor:%i", i+1);
            pSensorAdcFile[i] = fopen(dev_name, (const char*)&pList[i]);

            if(pSensorAdcFile[i] != NULL){
                printf("done\n");
                return_code = IO_OK;
            } else {
                printf("failed\n");
                break;
            }
        }
    }
    return return_code;
}


/*FUNCTION*****************************************************************
* 
* Function Name    : sensorAdcChanneClose
* Returned Value   : _mqx_int - IO_ERROR or IO_OK
* Comments         : Close Sensor Temperature(ADC) driver.
* 
*END*********************************************************************/
_mqx_int sensorAllChanneClose( void)
{
    _mqx_int i;

    printf("Closing channels...");

    for(i=0; i<SENSOR_FILE_NUM_MAX; i++){
        if(pSensorAdcFile[i] != NULL){
            fclose(pSensorAdcFile[i]);
        }
    }

    fclose(pSensorMainADC);
    printf("done\n");
    return SENSOR_OK;
}

/*FUNCTION*****************************************************************
* 
* Function Name    : sensorAdcMeasure
* Returned Value   : None

* Comments         : Opens and initializes Sensor (ADC) driver.
* 
*END*********************************************************************/
void sensorAdcMeasure( void)
{
    _mqx_int i;
    static SENSOR_ADC_RESULT_STRUCT data;
    SENSOR_ADC_VALUE_TYPE   nowAdc = 0;

    SENSOR_PARAM_PTR pParam = sensorGetParamAddress();

//    SENSOR_PRINTF(printf("\nADC "));

    for(i=0; i<SENSOR_MEATURE_NUM; i++){
        if (read(pSensorAdcFile[i], &data, sizeof(data) )){
            nowAdc = SensorMeasureData[i];
            SensorMeasureData[i] = data.result;
      
            SensorAdcDelta[i] = (_mqx_int)((_mqx_int)nowAdc - (_mqx_int)SensorMeasureData[i]);
//            SENSOR_PRINTF(printf("[%d]:%x, %d/ ", i, data.result, SensorAdcDelta[i]));
        }
    }

//    SENSOR_PRINTF(printf("\n"));
}


/*FUNCTION*****************************************************************
* 
* Function Name    : sensor_3axis_acceleration_change_check
* Input Value      : None
* Returned Value   : SENSOR_3AXIS_MOVEMENT (1): 3axis was operation
*                    0                        : 3axis was not operation
*                    less than 0              : Error
* Comments         : It is checked whether 
*                          there is movement for three axis sensor. 
* 
*END*********************************************************************/
_mqx_int sensor_3axis_acceleration_change_check( void)
{
//    static SENSOR_ADC_VALUE_TYPE amountValue[SENSOR_3AXIS_SIZE];
    static int_32 temp_SensorAdcDelta[4];
	/* ̕ω */
    static int_32 n_SensorAdcDelta[4];
    _mqx_int return_code;



#if 0

    /* SensorAdcDelta -> local Value */
    if(SensorAdcDelta[SENSOR_ID_X] < 0){
        amountValue[SENSOR_3AXIS_X] = (SENSOR_ADC_VALUE_TYPE)(-1 * SensorAdcDelta[SENSOR_ID_X]);
    } else {
        amountValue[SENSOR_3AXIS_X] = (SENSOR_ADC_VALUE_TYPE)(SensorAdcDelta[SENSOR_ID_X]);
    }

    if(SensorAdcDelta[SENSOR_ID_Y] < 0){
        amountValue[SENSOR_3AXIS_Y] = (SENSOR_ADC_VALUE_TYPE)(-1 * SensorAdcDelta[SENSOR_ID_Y]);
    } else {
        amountValue[SENSOR_3AXIS_Y] = (SENSOR_ADC_VALUE_TYPE)(SensorAdcDelta[SENSOR_ID_Y]);
    }

    if(SensorAdcDelta[SENSOR_ID_Z] < 0){
        amountValue[SENSOR_3AXIS_Z] = (SENSOR_ADC_VALUE_TYPE)(-1 * SensorAdcDelta[SENSOR_ID_Z]);
    } else {
        amountValue[SENSOR_3AXIS_Z] = (SENSOR_ADC_VALUE_TYPE)(SensorAdcDelta[SENSOR_ID_Z]);
    }

    if((Sensor3axisMode & SENSOR_3AXIS_MODE_Z_TYPE)){
        sensor_3axis_check_z_shake( (void*)amountValue[SENSOR_3AXIS_Z]);
    } else {

    /*  3axis Movement Function */
//    if((Sensor3axisMode & SENSOR_3AXIS_MODE_STRENGTH)){
        /* One axis multiple action */
//    } else {
        /* One axis one action */
        return_code = sensor_3axis_check_normal((void*)amountValue);
//      }
    }

#endif

	temp_SensorAdcDelta[SENSOR_ID_X] = 0;
	temp_SensorAdcDelta[SENSOR_ID_Y] = 0;
	temp_SensorAdcDelta[SENSOR_ID_Z] = 0;

	/* X ɐω_̃`^O */
	n_SensorAdcDelta[SENSOR_ID_X] = SensorAdcDelta[SENSOR_ID_X]/2;

	if ((n_SensorAdcDelta[SENSOR_ID_X]<=4)&&(n_SensorAdcDelta[SENSOR_ID_X]>=-4)){
		n_SensorAdcDelta[SENSOR_ID_X] = 0;
	}

	if (b_SensorAdcDelta[SENSOR_ID_X] != n_SensorAdcDelta[SENSOR_ID_X]){

		if ((b_SensorAdcDelta[SENSOR_ID_X]>0 && n_SensorAdcDelta[SENSOR_ID_X]>0)
		 ||(b_SensorAdcDelta[SENSOR_ID_X]<0 && n_SensorAdcDelta[SENSOR_ID_X]<0)){

			t_SensorAdcDelta[SENSOR_ID_X] = t_SensorAdcDelta[SENSOR_ID_X] + n_SensorAdcDelta[SENSOR_ID_X];

			if (SensorAdcDelta_count[SENSOR_ID_X] == 1){
				SensorAdcDelta_count[SENSOR_ID_X]++;
			}
			else{
				temp_SensorAdcDelta[SENSOR_ID_X] = t_SensorAdcDelta[SENSOR_ID_X];
			}
		}
		else{
			SensorAdcDelta_count[SENSOR_ID_X] = 0;
			t_SensorAdcDelta[SENSOR_ID_X] = 0;
		}
	}
	b_SensorAdcDelta[SENSOR_ID_X] = n_SensorAdcDelta[SENSOR_ID_X];


	/* Y ɐω_̃`^O */
	n_SensorAdcDelta[SENSOR_ID_Y] = SensorAdcDelta[SENSOR_ID_Y]/2;
	if ((n_SensorAdcDelta[SENSOR_ID_Y]<=4)&&(n_SensorAdcDelta[SENSOR_ID_Y]>=-4)){
		n_SensorAdcDelta[SENSOR_ID_Y] = 0;
	}

	if (b_SensorAdcDelta[SENSOR_ID_Y] != n_SensorAdcDelta[SENSOR_ID_Y]){

		if ((b_SensorAdcDelta[SENSOR_ID_Y]>0 && n_SensorAdcDelta[SENSOR_ID_Y]>0)
		 ||(b_SensorAdcDelta[SENSOR_ID_Y]<0 && n_SensorAdcDelta[SENSOR_ID_Y]<0)){

			t_SensorAdcDelta[SENSOR_ID_Y] = t_SensorAdcDelta[SENSOR_ID_Y] + n_SensorAdcDelta[SENSOR_ID_Y];

			if (SensorAdcDelta_count[SENSOR_ID_Y] == 1){
				SensorAdcDelta_count[SENSOR_ID_Y]++;
			}
			else{
				temp_SensorAdcDelta[SENSOR_ID_Y] = t_SensorAdcDelta[SENSOR_ID_Y];
			}
		}
		else{
			SensorAdcDelta_count[SENSOR_ID_Y] = 0;
			t_SensorAdcDelta[SENSOR_ID_Y] = 0;
		}
	}
	b_SensorAdcDelta[SENSOR_ID_Y] = n_SensorAdcDelta[SENSOR_ID_Y];


	/* Z ɐω_̃`^O */
	n_SensorAdcDelta[SENSOR_ID_Z] = SensorAdcDelta[SENSOR_ID_Z]/2;

	if ((n_SensorAdcDelta[SENSOR_ID_Z]<=10)&&(n_SensorAdcDelta[SENSOR_ID_Z]>=-10)){
		n_SensorAdcDelta[SENSOR_ID_Z] = 0;
	}

	if (b_SensorAdcDelta[SENSOR_ID_Z] != n_SensorAdcDelta[SENSOR_ID_Z]){

		if ((b_SensorAdcDelta[SENSOR_ID_Z]>0 && n_SensorAdcDelta[SENSOR_ID_Z]>0)
		 ||(b_SensorAdcDelta[SENSOR_ID_Z]<0 && n_SensorAdcDelta[SENSOR_ID_Z]<0)){

			t_SensorAdcDelta[SENSOR_ID_Z] = t_SensorAdcDelta[SENSOR_ID_Z] + n_SensorAdcDelta[SENSOR_ID_Z];

			if (SensorAdcDelta_count[SENSOR_ID_Z] == 1){
				SensorAdcDelta_count[SENSOR_ID_Z]++;
			}
			else{
				temp_SensorAdcDelta[SENSOR_ID_Z] = t_SensorAdcDelta[SENSOR_ID_Z];
			}
		}
		else{
			SensorAdcDelta_count[SENSOR_ID_Z] = 0;
			t_SensorAdcDelta[SENSOR_ID_Z] = 0;
		}
	}
	b_SensorAdcDelta[SENSOR_ID_Z] = n_SensorAdcDelta[SENSOR_ID_Z];


	if ((temp_SensorAdcDelta[SENSOR_ID_X] != 0)||(temp_SensorAdcDelta[SENSOR_ID_Y] != 0)||(temp_SensorAdcDelta[SENSOR_ID_Z] != 0)){
		/* X,Y,Z̕ω B*/
        return_code = sensor_3axis_check_normal((void*)temp_SensorAdcDelta);
	}

    return return_code;
}




/*FUNCTION*****************************************************************
* 
* Function Name    : sensor_3axis_check_normal
* Input Value      : [pData] Absolute amount of change of Adc value
*
* Returned Value   : SENSOR_3AXIS_MOVEMENT (1): 3axis was operation
*                    0                        : 3axis was not operation
*                    less than 0              : Error
* Comments         : 3 axis Move Check.
* 
*END*********************************************************************/
_mqx_int sensor_3axis_check_normal( void* pData)
{
//    SENSOR_ADC_VALUE_TYPE *pAmount = NULL;
    int_32 *pAmount = NULL;
    _mqx_int result = SENSOR_OK;
    SENSOR_PARAM_PTR pParam = sensorGetParamAddress();
//    SENSOR_ADC_VALUE_TYPE checkValue;
//    _mqx_int axis;
	int_32 para_3AXIS;
	int_32 level_3AXIS;

	/* m̑Εω */
	static int_32 temp_SensorAdc[4];

    pAmount = (int_32*)pData;

    if(pAmount == NULL){
        return SENSOR_ERR_INVALID_ARG;
    }

	/* ΕωʂZo */
	temp_SensorAdc[SENSOR_ID_X] = (pAmount[SENSOR_ID_X]<0)?  (pAmount[SENSOR_ID_X]*(-1)) : pAmount[SENSOR_ID_X];
	temp_SensorAdc[SENSOR_ID_Y] = (pAmount[SENSOR_ID_Y]<0)?  (pAmount[SENSOR_ID_Y]*(-1)) : pAmount[SENSOR_ID_Y];
	temp_SensorAdc[SENSOR_ID_Z] = (pAmount[SENSOR_ID_Z]<0)?  (pAmount[SENSOR_ID_Z]*(-1)) : pAmount[SENSOR_ID_Z];

	/* Εωʂ 0|Cg̋e͈͂Zo */
	temp_SensorAdc[SENSOR_ID_X] = (temp_SensorAdc[SENSOR_ID_X]<=10)?  0:temp_SensorAdc[SENSOR_ID_X];
	temp_SensorAdc[SENSOR_ID_Y] = (temp_SensorAdc[SENSOR_ID_Y]<=30)?  0:temp_SensorAdc[SENSOR_ID_Y];

	if ((temp_SensorAdc[SENSOR_ID_X] != 0) || (temp_SensorAdc[SENSOR_ID_Y] != 0)){
		if (temp_SensorAdc[SENSOR_ID_X]>temp_SensorAdc[SENSOR_ID_Y]){
			printf("x L[%d] \n", pAmount[SENSOR_ID_X]/20);
			para_3AXIS = ((pAmount[SENSOR_ID_X])>0)?  0 : 1;
			level_3AXIS = temp_SensorAdc[SENSOR_ID_X]/20;
		}
		else{
			printf("y L[%d] \n", pAmount[SENSOR_ID_Y]/20);
			para_3AXIS = ((pAmount[SENSOR_ID_Y])>0)?  2 : 3;
			level_3AXIS = temp_SensorAdc[SENSOR_ID_Y]/20;
		}

		SignalEvent_Change_Effect_Bank(para_3AXIS, level_3AXIS);
	}

#if 0
#ifndef SENSOR_ENABLE_EASY_DEMO
    /* Is the difference of which axis great? */
    if( pAmount[SENSOR_3AXIS_X] >= pAmount[SENSOR_3AXIS_Y] &&
        pAmount[SENSOR_3AXIS_X] >= pAmount[SENSOR_3AXIS_Z]){
        /* X axis is the largest.  */
        checkValue = pAmount[SENSOR_3AXIS_X];
        axis = SENSOR_3AXIS_X;
    } else {
        if( pAmount[SENSOR_3AXIS_Y] < pAmount[SENSOR_3AXIS_Z]){
            /* Z axis is the largest.  */
            checkValue = pAmount[SENSOR_3AXIS_Z];
            axis = SENSOR_3AXIS_Z;
        } else {
            /* Y axis is the largest.  */
            checkValue = pAmount[SENSOR_3AXIS_Y];
            axis = SENSOR_3AXIS_Y;
        }
    }

    if(pParam->act_param[axis].threshold <= checkValue){

//        if( SensorOldEffectChange != pParam->act_param[axis].soundType ){
            SENSOR_PRINTF(printf("[Sensor %d] EffectType = %d -> %d/ threshold= %d / diff = %d\n", 
                                               axis,
                                               SensorOldEffectChange,
                                               pParam->act_param[axis].soundType, 
                                               pParam->act_param[axis].threshold, 
                                               checkValue));

            //result = SignalEvent_Set_Effect_Type(pParam->act_param[axis].soundType);
            result = SignalEvent_Change_Effect_Bank(pParam->act_param[axis].soundType);
            
            if(result != MQX_OK){
                printf("[Sensor] Error (No Effect)\n");
                result = SENSOR_ERR_SET_EVENT;
            } else {
                result = SENSOR_3AXIS_MOVEMENT;
                SensorOldEffectChange = pParam->act_param[axis].soundType;
            }
//        }
    }
#else
    {
        _mqx_int i;
        
        for(i=0; i<SENSOR_3AXIS_SIZE; i++){
            if(pAmount[i] > SensorDemoThreshold){
                SignalEvent_Toggle_Effect_Bank();
                //SignalEvent_Change_Effect_Bank( SensorToggleParam);
                //SensorToggleParam++;
                //
                //if(SensorToggleParam>2){
                //  SensorToggleParam=0;
                //}
                
                result = SENSOR_3AXIS_MOVEMENT;
                break;
            }
        }
    }
#endif
#endif

    return result;
}

/*FUNCTION*****************************************************************
* 
* Function Name    : sensor_3axis_check_z_shake
* Input Value      : [pData] Absolute amount of change of Adc value
*
* Returned Value   : SENSOR_3AXIS_MOVEMENT (1): 3axis was operation
*                    0                        : 3axis was not operation
*                    less than 0              : Error
* Comments         : 3 axis Move Check.
* 
*END*********************************************************************/
_mqx_int sensor_3axis_check_z_shake( void *pData)
{
//    _mqx_int effectType = SIGNAL_EFFECT_TYPE_NORMAL;

//    SENSOR_ADC_VALUE_TYPE *pAmount = NULL;

//    if(pAmount[SENSOR_3AXIS_Z] >

    return 0;
}


/*TASK*-----------------------------------------------------
** 
** Task Name    : Sensor_task
** Comments     :
**
*END*-----------------------------------------------------*/

void Sensor_task
   (
      uint_32 initial_data
   )
{
    _mqx_int  result = 0;

    printf("Opening Sensor ADC device ...");

    /* Sensor Mian Open */
    result = sensorMainOpen();
    if(result != IO_OK){    
        printf("failed\n");
        _task_block();
    }

    /* Sensor Adc List Open */
    result = sensorAdcChannelOpen(&Sensor_adc_channel_param_list, SENSOR_ADC_LIST_CNANNEL_NUM_LIST);
    if(result != IO_OK){    
        printf("failed\n");
        _task_block();
    }

    sensorParamInit();
#ifdef SENSOR_DEBUG
    SensorEnableDebugMode = FALSE;
#endif

    SensorOldEffectChange = SIGNAL_EFFECT_TYPE_NORMAL;
    SensorWaitCnt = 0;
    SensorStatus = SENSOR_ST_INIT;
    SensorEnableChangeMode = FALSE;
    //SensorToggleParam = 0;

#ifdef SENSOR_ENABLE_EASY_DEMO
    SensorDemoThreshold = SENSOR_DEMO_ADC_THRESHOLD;
#endif

    SENSOR_INIT_DELAY();

    /* Sensor Main */
    while(1){

        /* ALL ADC Measurement */
        sensorAdcMeasure();

        sensorChangeModeFunc();

        sensor_status_func();
        SENSOR_DELAY(SENSOR_DELAY_TIME);
    }
    _mqx_exit(0);
}

/* EOF */
