/**HEADER********************************************************************
* 
* Copyright (c) 2010 Avnet Japan Co.,Ltd (NIKE Borad Project.)
* All Rights Reserved
*
**************************************************************************
*
* $FileName: Signal_Shell_Commands.c$
* $Version : 0.0.8.0$
* $Date    : Dec-5-2009$
*
* Comments:
*
*
*
*END************************************************************************/


#include "NIKE_demo.h"
#include <mqx.h>
#include <string.h>
#include <stdlib.h>
#include <shell.h>
#include "NIKE_demo_Shell_Commands.h"
#include "Signal_Task.h"


#define SIGNAL_FUNCTION_DEBUG      0

#define EFFECT_COMMAND_ILLEGAL_PRINTF()  printf("Invalid command.  Type 'help' for a list of commands.\n")



static int_32  Shell_effect_test(int_32 argc, char_ptr argv[] );
static int_32  Shell_effect_change_distortion_gain(int_32 argc, char_ptr argv[] );
static int_32  Shell_effect_change_distortion_range(int_32 argc, char_ptr argv[] );
static int_32  Shell_effect_change_fuzz_gain(int_32 argc, char_ptr argv[] );
static int_32  Shell_effect_change_fuzz_range(int_32 argc, char_ptr argv[] );
//static int_32  Shell_effect_change_echo_gain(int_32 argc, char_ptr argv[] );
static int_32  Shell_effect_change_echo_delay(int_32 argc, char_ptr argv[] );
static int_32  Shell_effect_change_type(int_32 argc, char_ptr argv[] );
static int_32  Shell_effect_change_bank(int_32 argc, char_ptr argv[] );
static int_32  Shell_effect_set_bank(int_32 argc, char_ptr argv[] );
static int_32  Shell_effect_show_params(int_32 argc, char_ptr argv[] );


const SHELL_COMMAND_STRUCT Shell_signal_shell_commands[] = {
#if SIGNAL_FUNCTION_DEBUG
    { "test",    Shell_effect_test                    },
    { "dgain",   Shell_effect_change_distortion_gain  },
    { "drange",  Shell_effect_change_distortion_range },
    { "fgain",   Shell_effect_change_fuzz_gain        },
    { "frange",  Shell_effect_change_fuzz_range       },
  //{ "egain",   Shell_effect_change_echo_gain        },
    { "edelay",  Shell_effect_change_echo_delay       },
    { "type",    Shell_effect_change_type             },
    { "bank",    Shell_effect_change_bank             },
    { "bset",    Shell_effect_set_bank                },
    { "show",    Shell_effect_show_params             },
#endif
    { NULL,   NULL }
};



/*FUNCTION*****************************************************************
* 
* Function Name    : Shell_effect_command
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
int_32  Shell_effect_command(int_32 argc, char_ptr argv[] )
{
    boolean           print_usage, shorthelp = FALSE;
    int_32            return_code = SHELL_EXIT_SUCCESS;
    uint_32           i;
    
    
    print_usage = Shell_check_help_request(argc, argv, &shorthelp );
    
    if(!print_usage){
       if(argc>1){
          for (i=0;Shell_signal_shell_commands[i].COMMAND != NULL;i++){
             if (strcmp(argv[1], Shell_signal_shell_commands[i].COMMAND) == 0){
                return_code = (*Shell_signal_shell_commands[i].SHELL_FUNC)(argc, argv);
                break;
             }
          }
          if (Shell_signal_shell_commands[i].COMMAND == NULL)  {
              EFFECT_COMMAND_ILLEGAL_PRINTF();
          }
      }
      else{
          //printf("c= %d, arg= %s, %s, %s\n", argc, argv[0], argv[1], argv[2]);
      }
   }
   else{
       //
   }
   return return_code;
} 



/*FUNCTION*****************************************************************
* 
* Function Name    : Shell_effect_test
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
static int_32  Shell_effect_test(int_32 argc, char_ptr argv[] )
{
    boolean print_usage, shorthelp = FALSE;
    int_32 return_code = SHELL_EXIT_SUCCESS;
    
    
    print_usage = Shell_check_help_request(argc, argv, &shorthelp );
    
    printf("<Signal Effect>  c=[%d], arg=[%s], [%s], [%s]\n", argc, argv[0], argv[1], argv[2]);
    
    return_code = SignalEffectStopAcceleration(0);
    
    return IO_OK;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : Shell_effect_change_distortion_gain
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
static int_32  Shell_effect_change_distortion_gain(int_32 argc, char_ptr argv[] )
{
    //boolean print_usage, shorthelp = FALSE;
    int_32  return_code;
    int_32 gain_val = 0;
    
    
    //print_usage = Shell_check_help_request(argc, argv, &shorthelp );
    
    //printf("<Signal Effect>  c=[%d], arg=[%s], [%s], [%s]\n", argc, argv[0], argv[1], argv[2]);
    
    gain_val = atoi(argv[2]);
    
    return_code = effect_change_distortion_gain(gain_val);
    if ( return_code != IO_OK ){
        return_code = SHELL_EXIT_ERROR;
    }
    else{
        return_code = SHELL_EXIT_SUCCESS;
    }
    return return_code;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : Shell_effect_change_distortion_range
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
static int_32  Shell_effect_change_distortion_range(int_32 argc, char_ptr argv[] )
{
    //boolean print_usage, shorthelp = FALSE;
    int_32 return_code = SHELL_EXIT_SUCCESS;
    int_32 range_val = 0;
    
    
    //print_usage = Shell_check_help_request(argc, argv, &shorthelp );
    
    //printf("<Signal Effect>  c=[%d], arg=[%s], [%s], [%s]\n", argc, argv[0], argv[1], argv[2]);
    
    range_val = atoi(argv[2]);
    
    return_code = effect_change_distortion_range(range_val);
    if ( return_code != IO_OK ){
        return_code = SHELL_EXIT_ERROR;
    }
    else{
        return_code = SHELL_EXIT_SUCCESS;
    }
    return return_code;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : Shell_effect_change_fuzz_gain
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
static int_32  Shell_effect_change_fuzz_gain(int_32 argc, char_ptr argv[] )
{
    //boolean print_usage, shorthelp = FALSE;
    int_32  return_code;
    int_32 gain_val = 0;
    
    
    //print_usage = Shell_check_help_request(argc, argv, &shorthelp );
    
    //printf("<Signal Effect>  c=[%d], arg=[%s], [%s], [%s]\n", argc, argv[0], argv[1], argv[2]);
    
    gain_val = atoi(argv[2]);
    
    return_code = effect_change_fuzz_gain(gain_val);
    if ( return_code != IO_OK ){
        return_code = SHELL_EXIT_ERROR;
    }
    else{
        return_code = SHELL_EXIT_SUCCESS;
    }
    return return_code;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : Shell_effect_change_fuzz_range
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
static int_32  Shell_effect_change_fuzz_range(int_32 argc, char_ptr argv[] )
{
    //boolean print_usage, shorthelp = FALSE;
    int_32 return_code = SHELL_EXIT_SUCCESS;
    int_32 range_val = 0;
    
    
    //print_usage = Shell_check_help_request(argc, argv, &shorthelp );
    
    //printf("<Signal Effect>  c=[%d], arg=[%s], [%s], [%s]\n", argc, argv[0], argv[1], argv[2]);
    
    range_val = atoi(argv[2]);
    
    return_code = effect_change_fuzz_range(range_val);
    if ( return_code != IO_OK ){
        return_code = SHELL_EXIT_ERROR;
    }
    else{
        return_code = SHELL_EXIT_SUCCESS;
    }
    return return_code;
}



#if 0
/*FUNCTION*****************************************************************
* 
* Function Name    : Shell_effect_change_echo_gain
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
static int_32  Shell_effect_change_echo_gain(int_32 argc, char_ptr argv[] )
{
    //boolean print_usage, shorthelp = FALSE;
    int_32  return_code;
    int_32 gain_val = 0;
    
    
    //print_usage = Shell_check_help_request(argc, argv, &shorthelp );
    
    //printf("<Signal Effect>  c=[%d], arg=[%s], [%s], [%s]\n", argc, argv[0], argv[1], argv[2]);
    
    gain_val = atoi(argv[2]);
    
    return_code = effect_change_echo_gain(gain_val);
    
    if (return_code != MQX_OK){
        return_code = SHELL_EXIT_ERROR;
    }
    else{
        return_code = SHELL_EXIT_SUCCESS;
    }
    return return_code;
}


#endif
/*FUNCTION*****************************************************************
* 
* Function Name    : Shell_effect_change_echo_gain
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
static int_32  Shell_effect_change_echo_delay(int_32 argc, char_ptr argv[] )
{
    //boolean print_usage, shorthelp = FALSE;
    int_32  return_code;
    int_32 delay_val = 0;
    
    
    //print_usage = Shell_check_help_request(argc, argv, &shorthelp );
    
    //printf("<Signal Effect>  c=[%d], arg=[%s], [%s], [%s]\n", argc, argv[0], argv[1], argv[2]);
    
    delay_val = atoi(argv[2]);
    
    return_code = effect_change_echo_delay(delay_val);
    
    if (return_code != MQX_OK){
        return_code = SHELL_EXIT_ERROR;
    }
    else{
        return_code = SHELL_EXIT_SUCCESS;
    }
    return return_code;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : Shell_effect_change_type
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
static int_32  Shell_effect_change_type(int_32 argc, char_ptr argv[] )
{
    //boolean print_usage, shorthelp = FALSE;
    int_32  return_code;
    int_32 effect_type = 0;
    
    
    //print_usage = Shell_check_help_request(argc, argv, &shorthelp );
    
    //printf("<Signal Effect>  c=[%d], arg=[%s], [%s], [%s]\n", argc, argv[0], argv[1], argv[2]);
    
    effect_type = atoi(argv[2]);
    
    return_code = SignalEvent_Set_Effect_Type(effect_type);
    
    if (return_code != MQX_OK){
        return_code = SHELL_EXIT_ERROR;
    }
    else{
        return_code = SHELL_EXIT_SUCCESS;
    }
    return return_code;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : Shell_effect_change_bank
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
static int_32  Shell_effect_change_bank(int_32 argc, char_ptr argv[] )
{
    //boolean print_usage, shorthelp = FALSE;
    int_32  return_code;
    int_32 effect_bank = 0;
    
    
    //print_usage = Shell_check_help_request(argc, argv, &shorthelp );
    
    //printf("<Signal Effect>  c=[%d], arg=[%s], [%s], [%s]\n", argc, argv[0], argv[1], argv[2]);
    
    effect_bank = atoi(argv[2]);
    
//    return_code = SignalEvent_Change_Effect_Bank(effect_bank);

	return_code = SignalEvent_Change_Effect_Bank(effect_bank, 0);

    if (return_code != MQX_OK){
        return_code = SHELL_EXIT_ERROR;
    }
    else{
        return_code = SHELL_EXIT_SUCCESS;
    }
    return return_code;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : Shell_effect_set_bank
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
static int_32  Shell_effect_set_bank(int_32 argc, char_ptr argv[] )
{
    //boolean print_usage, shorthelp = FALSE;
    int_32  return_code;
    int_32 effect_bank = 0;
    int_32 effect_first_type = 0;
    int_32 effect_second_type = 0;
    
    
    //print_usage = Shell_check_help_request(argc, argv, &shorthelp );
    
    //printf("<Signal Effect>  c=[%d], arg=[%s], [%s], [%s]\n", argc, argv[0], argv[1], argv[2]);
    
    effect_bank = atoi(argv[2]);
    effect_first_type = atoi(argv[3]);
    effect_second_type = atoi(argv[4]);
    
    return_code = effect_change_set_bank(effect_bank, effect_first_type, effect_second_type);
    
    if (return_code != MQX_OK){
        return_code = SHELL_EXIT_ERROR;
    }
    else{
        return_code = SHELL_EXIT_SUCCESS;
    }
    return return_code;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : Shell_effect_change_type
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
static int_32  Shell_effect_show_params(int_32 argc, char_ptr argv[] )
{
    //boolean print_usage, shorthelp = FALSE;
    
    
    //print_usage = Shell_check_help_request(argc, argv, &shorthelp );
    
    //printf("<Signal Effect>  c=[%d], arg=[%s], [%s], [%s]\n", argc, argv[0], argv[1], argv[2]);
    
    effect_show_params();
    
    return IO_OK;
}



