/**HEADER********************************************************************
* 
* Copyright (c) 2010 Avnet Japan Co.,Ltd (NIKE Borad Project.)
* All Rights Reserved
*
*************************************************************************** 
*
* $FileName: Shell_Task.c$
* $Version : 3.0.5.0$
* $Date    : Nov-21-2008$
*
* Comments:
*
*
*
*END************************************************************************/

#include "NIKE_demo.h"
#include "event.h"
#include "Signal_Task.h"
#include <AD7652.h>
#include <AD5546.h>
#include <dsp_library.h>


/* ------------------------------------------------------------------ 
   --- macro definition                                           --- 
   ------------------------------------------------------------------ */


#if NIKE_TESTMODE
#define SIGNAL_NORMAL_MODE		/* changed to normal mode */
#endif

#if NIKE_USB_DEVICE_MASS_STORAGE
#define SIGNAL_NORMAL_MODE		/* changed to normal mode */
#endif

/*---------- DEFINE ----------*/
//#define SIGNAL_NORMAL_MODE		/* changed to normal mode */
//#define NIKE_DEMO_EFFECT			/* GtFNgf 		*/
//#define NIKE_DEMO_PITCH			/* sb`Vtgf 	*/
#define NIKE_DEMO_REC				/* ^f 			*/
//#define NIKE_DEMO_PLAY_SPECIAL	/* Đf 			*/
//#define NIKE_DEMO_ADI				/* ADIIvV		*/


#define SIGNAL_EVENT_START_SIGNAL		__BIT(0)
#define SIGNAL_EVENT_END_SIGNAL			__BIT(1)
#define SIGNAL_EVENT_GET_AD7652			__BIT(2)
#define SIGNAL_EVENT_SET_EFFECT			__BIT(3)

#define SIGNAL_EVENT_RECODE_PLAY		__BIT(4)
#define SIGNAL_EVENT_RECODE_REC			__BIT(5)
#define SIGNAL_EVENT_RECODE_PLAY_STOP	__BIT(6)
#define SIGNAL_EVENT_RECODE_REC_STOP	__BIT(7)
#define SIGNAL_EVENT_RECODE_PLAY_COUNT	__BIT(8)


#ifdef SIGNAL_NORMAL_MODE
#define SIGNAL_BPS							100000L		/* 100KHz sampling */
#else
#define SIGNAL_BPS							50000L		/* 50KHz sampling */
#endif

//#define SIGNAL_DATA_COUNT					300
//#define SIGNAL_DATA_SIZE					(300*2)
#define SIGNAL_DATA_COUNT					((BSP_SIGNAL_DMA_TIME * SIGNAL_BPS)/1000)
#define SIGNAL_DATA_SIZE					(SIGNAL_DATA_COUNT*2)
#define EFFECT_BANK_MAX						4
#define EFFECT_STACK_MAX					2
#define EFFECT_ECHO_BUFFER_MAX				20

/* EFFECT - DISTORTION */
#define EFFECT_DISTORTION_GAIN_INIT_VAL		50
#define EFFECT_DISTORTION_RANGE_INIT_VAL	3000

/* EFFECT - FUZZ */
//#define EFFECT_FUZZ_GAIN_INIT_VAL			60
//#define EFFECT_FUZZ_RANGE_INIT_VAL		5000
#define EFFECT_FUZZ_GAIN_INIT_VAL			20
#define EFFECT_FUZZ_RANGE_INIT_VAL			3000

/* EFFECT - ECHO */
//#define EFFECT_ECHO_GAIN_INIT_VAL			10
#define EFFECT_ECHO_DELAY_INIT_VAL			20
#define EFFECT_ECHO_BUFFER_IN					0
#define EFFECT_ECHO_BUFFER_OUT					1


#define EFFECT_BUFFER_MAX					(EFFECT_ECHO_BUFFER_MAX*SIGNAL_DATA_SIZE)


#define EFFECT_LED1_CLR		BSP_LED1_CLR
#define EFFECT_LED2_CLR		BSP_LED2_CLR
#define EFFECT_LED1_SET		BSP_LED1_SET
#define EFFECT_LED2_SET		BSP_LED2_SET


#define RECODE_REC_MODE			1
#define RECODE_PLAY_MODE		2
#define RECODE_STOP_MODE		0


#define CHORUS_PARA          3
#define CHORUS_PARA_FS      (5000)										/* 50Hz */
#define CHORUS_PARA_HZ      (SIGNAL_BPS/CHORUS_PARA_FS)					/* (SIGNAL_BPS/CHORUS_PARA_FS) */
#define CHORUS_PARA_MAX     (CHORUS_PARA_HZ/2)
#define CHORUS_PARA_MIN		((CHORUS_PARA_HZ/2)*(-1))
//#define CHORUS_CAL(_A)      (-(CHORUS_PARA*CHORUS_PARA_MAX)+(CHORUS_PARA*_A))
#define CHORUS_CAL(_A)      (CHORUS_PARA * (-CHORUS_PARA_HZ+_A))


/* ------------------------------------------------------------------ 
   --- structure body definition                                  --- 
   ------------------------------------------------------------------ */



/* ------------------------------------------------------------------ 
   --- prototype declaration                                      --- 
   ------------------------------------------------------------------ */

_mqx_uint SignalEvent_Set_StartSignal(void);
_mqx_uint SignalEvent_Set_EndSignal(void);
_mqx_uint SignalEvent_Set_GetAD7652(void);
_mqx_uint SignalEvent_Set_Sensor(uchar sensorEvent);

void effect_set(void);
void effect_init(void);
void effect_normal_init(void);
void effect_normal(void *sig_buffer);
void effect_set_led(void);

//_mqx_int (*pEffect_func)(void*);
_mqx_int (*pEffect_func_first)(void*);
_mqx_int (*pEffect_func_second)(void*);
_mqx_int (*pEffect_buffer_func)(void*);

int  effect_recode_init_func(void);
int  effect_recode_Play_start_func(int number);
int  effect_recode_Play_stop_func(void);
int  effect_recode_Rec_start_func(int number);
int  effect_recode_Rec_stop_func(void);

void SwitchEvent_Set_PLAY_OFF(void);

int  effect_chorus_set_func(void);
int  effect_chorus_init_func(void);
void *memcpy(void *dest, const void *source, _mem_size n);

#ifndef SIGNAL_NORMAL_MODE
void effect_type_count(void);
void effect_echo_init(void);
void effect_echo_buffer_copy(void *sig_buffer);
void effect_echo_buffer_through(void *sig_buffer);
void effect_echo(void *sig_buffer);
void effect_distortion_init(void);
void effect_fuzz_init(void);
void effect_distortion(void *sig_buffer);
void effect_fuzz(void *sig_buffer);
int  effect_chorus_func(void *sig_buffer);
int  effect_pitch_shift_init_func(void);
int  effect_pitch_shift_func(void *sig_buffer);
#endif

int  lpf_init_func(void);
int  lpf_func(void *sig_buffer);

int  effect_pitch_shift_para_set_func(int pitch_para);

_mqx_int SignalEvent_Set_RECODE_PLAY_3AXIS(int num);
_mqx_int SignalEvent_Set_RECODE_PLAY(void);

/* ------------------------------------------------------------------ 
   --- variable definition                                        --- 
   ------------------------------------------------------------------ */

#ifndef SIGNAL_NORMAL_MODE

uchar effect_signal_buffer[EFFECT_BUFFER_MAX];
static _mqx_int echo_sel_in;
static _mqx_int echo_sel_out;

/* Table element is SIGNAL_EFFECT_TYPE */
const _mqx_int (*pEffect_func_tbl[])(void*) =
{
	effect_normal,
	effect_echo,
	effect_distortion,
	effect_fuzz,
	effect_chorus_func,
	effect_pitch_shift_func
};

#endif


/*---------- GLOBAL ----------*/
uchar recv_signal_buffer[SIGNAL_DATA_SIZE];

uchar effect_current_bank;

uchar effect_first_current_type;
uchar effect_second_current_type;
_mqx_int number_of_effect_type;

pointer SignalEvent_handle;

int recode_mode;
EFFECT_PARAMS effect_params;
EFFECT_PARAMS effect_params_tmp;
static int effect_sel_in;
static int chorus_cnt;
static int chorus_cnt_sel;

uint_16 effect_bank_stat[EFFECT_BANK_MAX][EFFECT_STACK_MAX];

int Rec_num_count;
int Play_num_count;



/*FUNCTION*****************************************************************
* 
* Function Name    : SignalEvent_Set_GetAD7652
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
_mqx_uint SignalEvent_Set_GetAD7652(void)
{
	_mqx_uint event_result;
	
	event_result = _event_set(SignalEvent_handle, SIGNAL_EVENT_GET_AD7652);
	
	if (event_result != MQX_OK) { 
		printf("[SIGNAL]Error:_event_set\n");
	}
	return event_result;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : SignalEvent_Set_StartSignal
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
_mqx_uint SignalEvent_Set_StartSignal(void)
{
	_mqx_uint event_result;
	
	event_result = _event_set(SignalEvent_handle, SIGNAL_EVENT_START_SIGNAL);
	
	if (event_result != MQX_OK) { 
		printf("[SIGNAL]Error:_event_set\n");
	}
	return event_result;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : SignalEvent_Set_EndSignal
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
_mqx_uint SignalEvent_Set_EndSignal(void)
{
	_mqx_uint event_result;
	
	event_result = _event_set(SignalEvent_handle, SIGNAL_EVENT_END_SIGNAL);
	
	if (event_result != MQX_OK) { 
		printf("[SIGNAL]Error:_event_set\n");
	}
	return event_result;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : SignalEvent_Set_Effect_Type
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
_mqx_int SignalEvent_Set_Effect_Type(_mqx_int effect_type)
{
	_mqx_uint event_result;
	
	
	if ( (0 <= effect_type) && (effect_type < number_of_effect_type) ){
		//effect_first_current_type = effect_type;
		effect_bank_stat[effect_current_bank][0] = effect_type;
	}
	else{
		printf("[SIGNAL]Error : invalid argument. [0 - %d]: [%d]\n", (number_of_effect_type-1), effect_type);
		return IO_ERROR;
	}
	event_result = _event_set(SignalEvent_handle, SIGNAL_EVENT_SET_EFFECT);
	
	if (event_result != MQX_OK){ 
		printf("[SIGNAL]Error : _event_set\n");
	}
	return (_mqx_int)event_result;
}

_mqx_int b_para_3AXIS = -1;


/*FUNCTION*****************************************************************
* 
* Function Name    : SignalEvent_Change_Effect_Bank
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
_mqx_int SignalEvent_Change_Effect_Bank(_mqx_int para_3AXIS, _mqx_int level_3AXIS)
{
	_mqx_uint event_result = 0;
	int bank_number;

#ifndef NIKE_DEMO_PLAY_SPECIAL
	if (b_para_3AXIS != para_3AXIS){
		b_para_3AXIS = para_3AXIS;
#endif

		bank_number = para_3AXIS;

#ifdef NIKE_DEMO_PITCH
		if (bank_number == 2){
			effect_pitch_shift_para_set_func(2);
		}
		else if (bank_number == 3){
			effect_pitch_shift_para_set_func(5);
		}
		SignalEvent_Set_RECODE_PLAY();
#endif

#ifdef NIKE_DEMO_REC
		SignalEvent_Set_RECODE_PLAY();
#endif


#ifdef NIKE_DEMO_PLAY_SPECIAL
		SignalEvent_Set_RECODE_PLAY_3AXIS(para_3AXIS);
#endif


#ifdef NIKE_DEMO_EFFECT
		if ( (0 <= bank_number) && (bank_number < EFFECT_BANK_MAX) ){
			effect_current_bank = bank_number;
		}
		else{
			printf("[SIGNAL]Error : invalid argument. [0 - %d]: [%d]\n", (EFFECT_BANK_MAX-1), bank_number);
			return IO_ERROR;
		}
		
		event_result = _event_set(SignalEvent_handle, SIGNAL_EVENT_SET_EFFECT);
		
		if (event_result != MQX_OK){ 
			printf("[SIGNAL]Error : _event_set\n");
		}
#endif

#ifndef NIKE_DEMO_PLAY_SPECIAL
	}
#endif

	return (_mqx_int)event_result;
}



#if 0

/*FUNCTION*****************************************************************
* 
* Function Name    : SignalEvent_Change_Effect_Bank
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
_mqx_int SignalEvent_Change_Effect_Bank(_mqx_int bank_number)
{
	_mqx_uint event_result;
	
	
	if ( (0 <= bank_number) && (bank_number < EFFECT_BANK_MAX) ){
		effect_current_bank = bank_number;
	}
	else{
		printf("[SIGNAL]Error : invalid argument. [0 - %d]: [%d]\n", (EFFECT_BANK_MAX-1), bank_number);
		return IO_ERROR;
	}
	
	event_result = _event_set(SignalEvent_handle, SIGNAL_EVENT_SET_EFFECT);
	
	if (event_result != MQX_OK){ 
		printf("[SIGNAL]Error : _event_set\n");
	}
	return (_mqx_int)event_result;
}


/*FUNCTION*****************************************************************
* 
* Function Name    : SignalEvent_Toggle_Effect_Bank
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
_mqx_int SignalEvent_Toggle_Effect_Bank(void)
{
	_mqx_uint event_result;
	
	if ( effect_current_bank < (EFFECT_BANK_MAX-1) ){
		effect_current_bank++;
	}
	else{
		effect_current_bank = 0;
	}
	
	event_result = _event_set(SignalEvent_handle, SIGNAL_EVENT_SET_EFFECT);
	
	if (event_result != MQX_OK){
		printf("[SIGNAL]Error : _event_set\n");
	}
	return (_mqx_int)event_result;
}
#endif


/*TASK*-----------------------------------------------------------------
*
* Function Name  : SignalEvent_Set_RECODE_PLAY
* Returned Value : None
* Comments       :
*
*END------------------------------------------------------------------*/

_mqx_int SignalEvent_Set_RECODE_PLAY(void)
{
	_mqx_uint event_result;

	event_result = _event_set(SignalEvent_handle, SIGNAL_EVENT_RECODE_PLAY);

	if (event_result != MQX_OK) { 
		printf("[SIGNAL]Error:_event_set\n");
	}

	return (_mqx_int)event_result;
}


/*TASK*-----------------------------------------------------------------
*
* Function Name  : SignalEvent_Set_RECODE_PLAY_3AXIS
* Returned Value : None
* Comments       :
*
*END------------------------------------------------------------------*/

_mqx_int SignalEvent_Set_RECODE_PLAY_3AXIS(int num)
{
	_mqx_uint event_result;

	Play_num_count = num;

#ifdef NIKE_DEMO_ADI
	switch(num){
		case 0:
			Play_num_count = 2;
			break;

		case 1:
			Play_num_count = 3;
			break;

		case 2:
			Play_num_count = 1;
			break;

		case 3:
			Play_num_count = 0;
			break;
	}

#endif

	event_result = _event_set(SignalEvent_handle, SIGNAL_EVENT_RECODE_PLAY_COUNT);

	if (event_result != MQX_OK) { 
		printf("[SIGNAL]Error:_event_set\n");
	}

	return (_mqx_int)event_result;
}



/*TASK*-----------------------------------------------------------------
*
* Function Name  : SignalEvent_Set_RECODE_REC
* Returned Value : none
* Comments       :
*
*END------------------------------------------------------------------*/

_mqx_int SignalEvent_Set_RECODE_REC(void)
{
	_mqx_uint event_result;

	event_result = _event_set(SignalEvent_handle, SIGNAL_EVENT_RECODE_REC);

	if (event_result != MQX_OK) { 
		printf("[SIGNAL]Error:_event_set\n");
	}

	return (_mqx_int)event_result;
}



/*TASK*-----------------------------------------------------------------
*
* Function Name  : SignalEvent_Set_RECODE_PLAY_STOP
* Returned Value : none
* Comments       :
*
*END------------------------------------------------------------------*/
_mqx_int SignalEvent_Set_RECODE_PLAY_STOP(void)
{
	_mqx_uint event_result;

	event_result = _event_set(SignalEvent_handle, SIGNAL_EVENT_RECODE_PLAY_STOP);

	if (event_result != MQX_OK) { 
		printf("[SIGNAL]Error:_event_set\n");
	}

	return (_mqx_int)event_result;
}


/*TASK*-----------------------------------------------------------------
*
* Function Name  : SignalEvent_Set_RECODE_REC_STOP
* Returned Value : none
* Comments       :
*
*END------------------------------------------------------------------*/
_mqx_int SignalEvent_Set_RECODE_REC_STOP(void)
{
	_mqx_uint event_result;

	event_result = _event_set(SignalEvent_handle, SIGNAL_EVENT_RECODE_REC_STOP);

	if (event_result != MQX_OK) { 
		printf("[SIGNAL]Error:_event_set\n");
	}

	return (_mqx_int)event_result;
}



/*TASK*****************************************************************
* 
* Function Name    : Signal_Task
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
void Signal_Task(uint_32 temp)
{
	_mqx_uint         event_result;
	static _mqx_uint  event_bit;
	static FILE_PTR   adc_fh_ptr;
	static FILE_PTR   dac_fh_ptr;
	static int_32     adc_signal_bps;
	static int_32     dac_signal_bps;

	printf("[SIGNAL]Start Signal_Task\n");

	effect_init();

	Rec_num_count = 0;
	Play_num_count = 0;

	/* create event - event.Event */
	event_result = _event_create("Signalevent.Event");
	if (event_result != MQX_OK) { 
		/* event event.Event not be created */
		printf("[SIGNAL]Signalevent.Event not be created. \n");
	}
	
	/* open connection to event event.Event */
	event_result = _event_open("Signalevent.Event", &SignalEvent_handle);
	if (event_result != MQX_OK) { 
		/* could not open event.Event1  */
		printf("[SIGNAL]could not open Signalevent.Event. \n");
	}
	
	adc_fh_ptr = (FILE_PTR)fopen("A7652:", BSP_DEFAULT_IO_OPEN_MODE);
	if (adc_fh_ptr == NULL){
		/* could not open A7652 */
		printf("[SIGNAL]could not open A7652 \n");
	}
	
	dac_fh_ptr = (FILE_PTR)fopen("A5546:", BSP_DEFAULT_IO_OPEN_MODE);
	if (dac_fh_ptr == NULL){
		/* could not open A5546 */
		printf("[SIGNAL]could not open A5546 \n");
	}

	ioctl(adc_fh_ptr, IO_IOCTL_AD7652_SET_CALLBACK, (pointer)&SignalEvent_Set_GetAD7652);


	adc_signal_bps = SIGNAL_BPS;
	ioctl(adc_fh_ptr, IO_IOCTL_AD7652_SET_BPS, (pointer)&adc_signal_bps);

	dac_signal_bps = SIGNAL_BPS;
	ioctl(dac_fh_ptr, IO_IOCTL_AD5546_SET_BPS, (pointer)&dac_signal_bps);


#if !NIKE_TESTMODE
#if !NIKE_USB_DEVICE_MASS_STORAGE
	SignalEvent_Set_StartSignal();
#endif
#endif

	lpf_init_func();

	/* Run the shell on the serial port */
	while(1){
		/* wait event - event.Event */
		event_result = _event_wait_any( SignalEvent_handle, 0x0f, 0);
		if (event_result != MQX_OK) { 
			/* wait  the event event.Event1 failed */
			printf("[SIGNAL]wait the ad7652event event.Event failed. \n");
		}

		_event_get_value( SignalEvent_handle, &event_bit);

		if ((event_bit & SIGNAL_EVENT_START_SIGNAL) != 0){
			_event_clear( SignalEvent_handle, SIGNAL_EVENT_START_SIGNAL);
			
			printf("[SIGNAL]Event SIGNAL_EVENT_START_SIGNAL. \n");
			
			ioctl(adc_fh_ptr, IO_IOCTL_AD7652_START, 0);
			_time_delay(5);
			ioctl(dac_fh_ptr, IO_IOCTL_AD5546_START, 0);
		}

		if ((event_bit & SIGNAL_EVENT_END_SIGNAL) != 0){
			_event_clear( SignalEvent_handle, SIGNAL_EVENT_END_SIGNAL);
			
			printf("[SIGNAL]Event SIGNAL_EVENT_END_SIGNAL. \n");
		}

		if ((event_bit & SIGNAL_EVENT_GET_AD7652) != 0){
			_event_clear( SignalEvent_handle, SIGNAL_EVENT_GET_AD7652);

			read(adc_fh_ptr, recv_signal_buffer, 1024);

#ifndef SIGNAL_NORMAL_MODE
			if (recode_mode == RECODE_REC_MODE){
				MfsControlEvent_Set_SetSignalData(recv_signal_buffer, 600);
				if (MfsControl_Get_file_mode() == 0){
					recode_mode = RECODE_STOP_MODE;
					printf("recode_mode = RECODE_STOP_MODE. \n");
				}
			}
			else if (recode_mode == RECODE_PLAY_MODE){
				MfsControlEvent_Set_GetSignalData(recv_signal_buffer, 600);
				if (MfsControl_Get_file_mode() == 0){
					recode_mode = RECODE_STOP_MODE;
					printf("recode_mode = RECODE_STOP_MODE. \n");
				}
				/* buffering for echo effect */
				(*pEffect_buffer_func)((void*)recv_signal_buffer);

				/* effect function */
				//(*pEffect_func)((void*)recv_signal_buffer);
				(*pEffect_func_first)((void*)recv_signal_buffer);
				(*pEffect_func_second)((void*)recv_signal_buffer);
			}
			else{
				/* buffering for echo effect */
				(*pEffect_buffer_func)((void*)recv_signal_buffer);
				
				/* effect function */
				//(*pEffect_func)((void*)recv_signal_buffer);
				(*pEffect_func_first)((void*)recv_signal_buffer);
				(*pEffect_func_second)((void*)recv_signal_buffer);
			}
#endif

			write(dac_fh_ptr, recv_signal_buffer, 1024);
		}

		if ((event_bit & SIGNAL_EVENT_SET_EFFECT) != 0){
			_event_clear( SignalEvent_handle, SIGNAL_EVENT_SET_EFFECT);
			effect_set();
			//printf("[SIGNAL]Event SIGNAL_EVENT_SET_EFFECT. \n");
		}

		if ((event_bit & SIGNAL_EVENT_RECODE_PLAY) != 0){
			_event_clear( SignalEvent_handle, SIGNAL_EVENT_RECODE_PLAY);

			effect_recode_Play_start_func(Play_num_count);
#ifdef NIKE_DEMO_PLAY_SPECIAL
			Play_num_count++;
			Play_num_count = (Play_num_count>=4)? 0 : Play_num_count;
#endif
		}

		if ((event_bit & SIGNAL_EVENT_RECODE_PLAY_COUNT) != 0){
			_event_clear( SignalEvent_handle, SIGNAL_EVENT_RECODE_PLAY_COUNT);

			effect_recode_Play_start_func(Play_num_count);
		}

		if ((event_bit & SIGNAL_EVENT_RECODE_REC) != 0){
			_event_clear( SignalEvent_handle, SIGNAL_EVENT_RECODE_REC);

			effect_recode_Rec_start_func(Rec_num_count);

#ifdef NIKE_DEMO_PLAY_SPECIAL
			Rec_num_count++;
			Rec_num_count = (Rec_num_count>=4)? 0 : Rec_num_count;
#endif

		}

		if ((event_bit & SIGNAL_EVENT_RECODE_PLAY_STOP) != 0){
			_event_clear( SignalEvent_handle, SIGNAL_EVENT_RECODE_PLAY_STOP);

			effect_recode_Play_stop_func();
		}

		if ((event_bit & SIGNAL_EVENT_RECODE_REC_STOP) != 0){
			_event_clear( SignalEvent_handle, SIGNAL_EVENT_RECODE_REC_STOP);

			effect_recode_Rec_stop_func();
		}

	}
}



#if 0
/*FUNCTION*****************************************************************
* 
* Function Name    : effect_set
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
void effect_set(void)
{
	effect_params = effect_params_tmp;

#ifndef SIGNAL_NORMAL_MODE
	if ( effect_first_current_type == SIGNAL_EFFECT_TYPE_ECHO || effect_first_current_type == SIGNAL_EFFECT_TYPE_CHORUS ){
		//printf("[SIGNAL]buffer ---> through\n");
		pEffect_buffer_func = effect_echo_buffer_through;
	}
	else{
		//printf("[SIGNAL]buffer ---> copy\n");
		pEffect_buffer_func = effect_echo_buffer_copy;
	}

	pEffect_func_first = pEffect_func_tbl[effect_first_current_type];
	pEffect_func_second = pEffect_func_tbl[effect_second_current_type];

	if (effect_first_current_type == SIGNAL_EFFECT_TYPE_CHORUS ){
		effect_chorus_set_func();
	}
#endif

}
#endif


/*FUNCTION*****************************************************************
* 
* Function Name    : effect_set
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
void effect_set(void)
{
	effect_params = effect_params_tmp;

#ifndef SIGNAL_NORMAL_MODE
	switch(effect_bank_stat[effect_current_bank][0]){
		case SIGNAL_EFFECT_TYPE_ECHO:
		case SIGNAL_EFFECT_TYPE_CHORUS:
		case SIGNAL_EFFECT_TYPE_PT:
			//printf("[SIGNAL]buffer ---> through\n");
			pEffect_buffer_func = effect_echo_buffer_through;
			break;

		default:
			//printf("[SIGNAL]buffer ---> copy\n");
			pEffect_buffer_func = effect_echo_buffer_copy;
			break;
	}

	pEffect_func_first = pEffect_func_tbl[effect_bank_stat[effect_current_bank][0]];
	pEffect_func_second = pEffect_func_tbl[effect_bank_stat[effect_current_bank][1]];

	if (effect_bank_stat[effect_current_bank][0] == SIGNAL_EFFECT_TYPE_CHORUS ){
		effect_chorus_set_func();
	}
#endif

}


/*FUNCTION*****************************************************************
* 
* Function Name    : effect_init
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
void effect_init(void)
{
	effect_current_bank = 0;

#ifdef NIKE_DEMO_EFFECT
	effect_bank_stat[0][0] = SIGNAL_EFFECT_TYPE_NORMAL;
	effect_bank_stat[0][1] = SIGNAL_EFFECT_TYPE_NORMAL;
	
	effect_bank_stat[1][0] = SIGNAL_EFFECT_TYPE_CHORUS;
	effect_bank_stat[1][1] = SIGNAL_EFFECT_TYPE_NORMAL;
	
	effect_bank_stat[2][0] = SIGNAL_EFFECT_TYPE_ECHO;
	effect_bank_stat[2][1] = SIGNAL_EFFECT_TYPE_NORMAL;
	
	effect_bank_stat[3][0] = SIGNAL_EFFECT_TYPE_NORMAL;
	effect_bank_stat[3][1] = SIGNAL_EFFECT_TYPE_FUZZ;
#endif


#ifdef NIKE_DEMO_PITCH
	effect_bank_stat[0][0] = SIGNAL_EFFECT_TYPE_PT;
	effect_bank_stat[0][1] = SIGNAL_EFFECT_TYPE_NORMAL;

	effect_bank_stat[1][0] = SIGNAL_EFFECT_TYPE_NORMAL;
	effect_bank_stat[1][1] = SIGNAL_EFFECT_TYPE_NORMAL;

	effect_bank_stat[2][0] = SIGNAL_EFFECT_TYPE_NORMAL;
	effect_bank_stat[2][1] = SIGNAL_EFFECT_TYPE_NORMAL;

	effect_bank_stat[3][0] = SIGNAL_EFFECT_TYPE_NORMAL;
	effect_bank_stat[3][1] = SIGNAL_EFFECT_TYPE_NORMAL;
#endif

#ifdef NIKE_DEMO_REC
	effect_bank_stat[0][0] = SIGNAL_EFFECT_TYPE_NORMAL;
	effect_bank_stat[0][1] = SIGNAL_EFFECT_TYPE_NORMAL;

	effect_bank_stat[1][0] = SIGNAL_EFFECT_TYPE_NORMAL;
	effect_bank_stat[1][1] = SIGNAL_EFFECT_TYPE_NORMAL;

	effect_bank_stat[2][0] = SIGNAL_EFFECT_TYPE_NORMAL;
	effect_bank_stat[2][1] = SIGNAL_EFFECT_TYPE_NORMAL;

	effect_bank_stat[3][0] = SIGNAL_EFFECT_TYPE_NORMAL;
	effect_bank_stat[3][1] = SIGNAL_EFFECT_TYPE_NORMAL;
#endif


#ifdef NIKE_DEMO_PLAY_SPECIAL
	effect_bank_stat[0][0] = SIGNAL_EFFECT_TYPE_NORMAL;
	effect_bank_stat[0][1] = SIGNAL_EFFECT_TYPE_NORMAL;

	effect_bank_stat[1][0] = SIGNAL_EFFECT_TYPE_NORMAL;
	effect_bank_stat[1][1] = SIGNAL_EFFECT_TYPE_NORMAL;

	effect_bank_stat[2][0] = SIGNAL_EFFECT_TYPE_NORMAL;
	effect_bank_stat[2][1] = SIGNAL_EFFECT_TYPE_NORMAL;

	effect_bank_stat[3][0] = SIGNAL_EFFECT_TYPE_NORMAL;
	effect_bank_stat[3][1] = SIGNAL_EFFECT_TYPE_NORMAL;
#endif


#ifdef SIGNAL_NORMAL_MODE
	number_of_effect_type = 1;
	effect_normal_init();
#else
	effect_type_count();
	
	effect_normal_init();
	effect_echo_init();
	effect_distortion_init();
	effect_fuzz_init();
	effect_chorus_init_func();
	effect_pitch_shift_init_func();
#endif
	effect_set();

	effect_recode_init_func();
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_normal_init
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
void effect_normal_init(void)
{
	return;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_normal
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
void effect_normal(void *sig_buffer)
{
	return;
}



#ifdef SIGNAL_NORMAL_MODE

#else
/*FUNCTION*****************************************************************
* 
* Function Name    : effect_type_count
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
void effect_type_count(void)
{
	number_of_effect_type = sizeof(pEffect_func_tbl) / sizeof(*pEffect_func_tbl);
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_echo_init
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
void effect_echo_init(void)
{
	_mem_zero(effect_signal_buffer, sizeof(effect_signal_buffer));
	
	//effect_echo_gain_val = EFFECT_ECHO_GAIN_INIT_VAL;
	//effect_echo_gain_val_tmp = EFFECT_ECHO_GAIN_INIT_VAL;
	effect_params.echo_delay_val = EFFECT_ECHO_DELAY_INIT_VAL;
	effect_params_tmp.echo_delay_val = EFFECT_ECHO_DELAY_INIT_VAL;
	
	echo_sel_in  = EFFECT_ECHO_BUFFER_IN;
	echo_sel_out = EFFECT_ECHO_BUFFER_OUT;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_echo_buffer_copy
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
void  effect_echo_buffer_copy(void *sig_buffer)
{
	static int_16 *delay_buffer_in;
	static int_16 *delay_buffer_out;
	static int_16 *pecho_out;
	_mqx_int i;
	
	
	delay_buffer_in  = (int_16 *)&effect_signal_buffer[(echo_sel_in*SIGNAL_DATA_SIZE)];
	delay_buffer_out = (int_16 *)&effect_signal_buffer[(echo_sel_out*SIGNAL_DATA_SIZE)];
	pecho_out = (int_16 *)sig_buffer;
	
	for ( i=0; i < SIGNAL_DATA_COUNT; i++ ){
		delay_buffer_in[i] = (pecho_out[i]/2)+(delay_buffer_out[i]/2);
	}
	echo_sel_in = (echo_sel_in+1) % effect_params.echo_delay_val;
	
	if ( (echo_sel_in+1) >= effect_params.echo_delay_val ){
		echo_sel_out = 0;
	}
	else{
		echo_sel_out = echo_sel_in + 1;
	}
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_echo_buffer_through
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
void  effect_echo_buffer_through(void *sig_buffer)
{
	return;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_echo
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
void effect_echo(void *sig_buffer)
{
	static int_16 *delay_buffer_in;
	static int_16 *delay_buffer_out;
	static int_16 *pecho_out;
	static int_32 comp_data_tmp;
	static int_32 comp_data_out_tmp;
	_mqx_int i;

	delay_buffer_in  = (int_16 *)&effect_signal_buffer[(echo_sel_in*SIGNAL_DATA_SIZE)];
	delay_buffer_out = (int_16 *)&effect_signal_buffer[(echo_sel_out*SIGNAL_DATA_SIZE)];
	pecho_out = (int_16 *)sig_buffer;

	for ( i=0; i < SIGNAL_DATA_COUNT; i++ ){
#if 0
		comp_data_tmp = (pecho_out[i]/2)+(delay_buffer_out[i]/2);
		delay_buffer_in[i] = comp_data_tmp;
		pecho_out[i] = comp_data_tmp * (effect_echo_gain_val/10);
#endif

		comp_data_tmp = (int_32)pecho_out[i]+(int_32)delay_buffer_out[i];
		comp_data_tmp = comp_data_tmp/2;
		comp_data_out_tmp = comp_data_tmp;
		comp_data_out_tmp = (comp_data_out_tmp * 120L)/100L;
		comp_data_tmp = (comp_data_tmp * 180L)/100L;

		delay_buffer_in[i] = (int_16)comp_data_tmp;

		//pecho_out[i] = (int_16)(comp_data_tmp) * (effect_echo_gain_val/10);

		pecho_out[i] = (int_16)(comp_data_out_tmp);
	}
	echo_sel_in = (echo_sel_in+1) % effect_params.echo_delay_val;
	
	if ( (echo_sel_in+1) >= effect_params.echo_delay_val ){
		echo_sel_out = 0;
	}
	else{
		echo_sel_out = echo_sel_in + 1;
	}
	
#if 0	
	static int_16 *delay_buffer_in;
	static int_16 *delay_buffer_out;
	static int_16 *pecho_out;
	//int_16 comp_data_tmp;
	static int_32 comp_data_tmp;
	_mqx_int i;
	
	
	delay_buffer_in  = (int_16 *)&effect_signal_buffer[(echo_sel_in*SIGNAL_DATA_SIZE)];
	delay_buffer_out = (int_16 *)&effect_signal_buffer[(echo_sel_out*SIGNAL_DATA_SIZE)];
	pecho_out = (int_16 *)sig_buffer;
	
	for ( i=0; i < SIGNAL_DATA_COUNT; i++ ){
#if 0
		comp_data_tmp = (pecho_out[i]/2)+(delay_buffer_out[i]/2);
		delay_buffer_in[i] = comp_data_tmp;
		pecho_out[i] = comp_data_tmp * (effect_echo_gain_val/10);
#endif
		comp_data_tmp = (int_32)pecho_out[i]+(int_32)delay_buffer_out[i];
		comp_data_tmp = comp_data_tmp/2;
		delay_buffer_in[i] = (int_16)comp_data_tmp;
		//pecho_out[i] = (int_16)(comp_data_tmp) * (effect_echo_gain_val/10);
		pecho_out[i] = (int_16)(comp_data_tmp);
	}
	echo_sel_in = (echo_sel_in+1) % effect_params.echo_delay_val;
	
	if ( (echo_sel_in+1) >= effect_params.echo_delay_val ){
		echo_sel_out = 0;
	}
	else{
		echo_sel_out = echo_sel_in + 1;
	}
#endif

}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_distortion_init
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
void effect_distortion_init(void)
{
	effect_params.distortion_gain_val = EFFECT_DISTORTION_GAIN_INIT_VAL;
	effect_params.distortion_range_max = EFFECT_DISTORTION_RANGE_INIT_VAL;
	effect_params.distortion_range_min = effect_params.distortion_range_max * (-1);
	
	effect_params_tmp = effect_params;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_distortion
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
void effect_distortion(void *sig_buffer)
{
	_mqx_int i;
	int_16_ptr input_sound_data_p;
	_mqx_int sound_data_tmp;
	
	
	input_sound_data_p = (int_16 _PTR_)sig_buffer;
	
	for ( i=0; i < SIGNAL_DATA_COUNT; i++ ){
		
		sound_data_tmp = (_mqx_int)input_sound_data_p[i] * effect_params.distortion_gain_val / 10;
		
		if ( sound_data_tmp > effect_params.distortion_range_max ){
			input_sound_data_p[i] = effect_params.distortion_range_max;
		}
		else if ( sound_data_tmp < effect_params.distortion_range_min){
			input_sound_data_p[i] = effect_params.distortion_range_min;
		}
		else{
			input_sound_data_p[i] = (int_16)sound_data_tmp;
		}
	}
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_fuzz_init
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
void effect_fuzz_init(void)
{
	effect_params.fuzz_gain_val = EFFECT_FUZZ_GAIN_INIT_VAL;
	effect_params.fuzz_range_max = EFFECT_FUZZ_RANGE_INIT_VAL;
	effect_params.fuzz_range_min = EFFECT_FUZZ_RANGE_INIT_VAL * (-1);
	
	effect_params_tmp = effect_params;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_fuzz
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
void effect_fuzz(void *sig_buffer)
{
	_mqx_int i;
	int_16_ptr input_sound_data_p;
	_mqx_int sound_data_tmp;
	
	
	input_sound_data_p = (int_16 _PTR_)sig_buffer;
	
	for ( i=0; i < SIGNAL_DATA_COUNT; i++ ){
		if ( input_sound_data_p[i] < 0 ){
			input_sound_data_p[i] *= -1;
		}
		sound_data_tmp = (_mqx_int)input_sound_data_p[i] * effect_params.fuzz_gain_val / 10;
		
		if ( sound_data_tmp > effect_params.fuzz_range_max ){
			input_sound_data_p[i] = effect_params.fuzz_range_max;
		}
		else if ( sound_data_tmp < effect_params.fuzz_range_min){
			input_sound_data_p[i] = effect_params.fuzz_range_min;
		}
		else{
			input_sound_data_p[i] = (int_16)sound_data_tmp;
		}
	}
}


/* ------------------------------------------------------------------------------- */
/* -- Chorus Effect                                                                  */
/* ------------------------------------------------------------------------------- */


/*FUNCTION*****************************************************************
* 
* Function Name    : effect_chorus_init_func
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
int  effect_chorus_init_func(void)
{
	effect_sel_in  = 0;

	chorus_cnt = 0;
	chorus_cnt_sel = 0;

	return 0;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_chorus_set_func
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
int  effect_chorus_set_func(void)
{
	effect_sel_in  = 0;

	chorus_cnt = 0;
	chorus_cnt_sel = 0;
	return 0;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_chorus_func
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
int  effect_chorus_func(void *sig_buffer)
{
	static int_16 *peffect_in;
	static int_16 *peffect_out;
	int now_effect_sel_in;
	int chorus_sel_in;
	int i;

	now_effect_sel_in = (effect_sel_in * SIGNAL_DATA_SIZE);

	memcpy(&effect_signal_buffer[effect_sel_in*SIGNAL_DATA_SIZE], sig_buffer, SIGNAL_DATA_SIZE);
	effect_sel_in   = (effect_sel_in+1) % (EFFECT_ECHO_BUFFER_MAX);

	peffect_in   = (int_16 *)&effect_signal_buffer[0];
	peffect_out  = (int_16 *)sig_buffer;
	for (i=0;i<(SIGNAL_DATA_SIZE/2);i++){
		chorus_sel_in = (((now_effect_sel_in/2)+i)+CHORUS_CAL(chorus_cnt));

		peffect_out[i]  = (peffect_out[i]/2)  +(peffect_in[((0>chorus_sel_in)? ((EFFECT_BUFFER_MAX/2)+chorus_sel_in):(chorus_sel_in%(EFFECT_BUFFER_MAX/2)))]/2);

		if (chorus_cnt_sel == 0){
			chorus_cnt = chorus_cnt+1;
			if (chorus_cnt>=CHORUS_PARA_MAX){
				chorus_cnt_sel = 1;
			}
		}
		else{
			chorus_cnt = chorus_cnt-1;
			if (chorus_cnt<=CHORUS_PARA_MIN){
				chorus_cnt_sel = 0;
			}
		}
	}

	return 0;
}


const int pitch_level_tbl[23][21] = {
	{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /* 0%  */
	{  1, 50,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /* 2%  */
	{  1, 20,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /* 5%  */
	{  2, 25, 25,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /* 8%  */
	{  2, 10, 10,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /* 10%  */
	{  3,  7,  7,  6,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /* 15%  */
	{  4,  5,  5,  5,  5,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /* 20%  */
	{  5,  4,  4,  4,  4,  4,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /* 25%  */
	{  6,  4,  3,  3,  4,  3,  3,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /* 30%  */
	{  7,  3,  3,  3,  3,  3,  3,  2,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /* 35%  */
	{  8,  3,  2,  3,  2,  3,  2,  3,  2,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /* 40%  */
	{  9,  3,  2,  2,  2,  2,  3,  2,  2,  2,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /* 45%  */
	{ 10,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /* 50%  */
	{ 11,  2,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /* 55%  */
	{ 12,  2,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0 },  /* 60%  */
	{ 13,  2,  2,  2,  2,  2,  2,  2,  1,  1,  1,  1,  1,  1,  0,  0,  0,  0,  0,  0,  0 },  /* 65%  */
	{ 14,  2,  2,  2,  2,  2,  2,  1,  1,  1,  1,  1,  1,  1,  1,  0,  0,  0,  0,  0,  0 },  /* 70%  */
	{ 15,  2,  2,  2,  2,  2,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  0,  0,  0,  0,  0 },  /* 75%  */
	{ 16,  2,  2,  2,  2,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  0,  0,  0,  0 },  /* 80%  */
	{ 17,  2,  2,  2,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  0,  0,  0 },  /* 85%  */
	{ 18,  2,  2,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  0,  0 },  /* 90%  */
	{ 19,  2,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  0 },  /* 95%  */
	{ 20,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },  /* 100%  */
};

int pitch_level_point;
int pitch_level_max;
int pitch_level_count;

int pitch_buffer_count;
int pitch_out_count;

int pitch_level;
int pitch_level_temp;

int pitch_time;
int pitch_time_sel;

int  effect_pitch_shift_set_func(void);


/*FUNCTION*****************************************************************
* 
* Function Name    : effect_pitch_shift_init_func
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
int  effect_pitch_shift_init_func(void)
{
	pitch_level = 0;
	pitch_level_temp = 0;

	pitch_level_max = pitch_level_tbl[pitch_level][0];
	pitch_level_point = 0;
	pitch_level_count = pitch_level_tbl[pitch_level][pitch_level_point+1];

	pitch_buffer_count = 0;
	pitch_out_count = 0;

	pitch_time = 0;
	pitch_time_sel = 0;

	return 0;
}


/*FUNCTION*****************************************************************
* 
* Function Name    : effect_pitch_shift_set_func
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
int  effect_pitch_shift_set_func(void)
{
	pitch_level = pitch_level_temp;

	return 0;
}


int  effect_pitch_shift_down_set_func(void);
int  effect_pitch_shift_up_set_func(void);


/*FUNCTION*****************************************************************
* 
* Function Name    : effect_pitch_shift_up_set_func
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
int  effect_pitch_shift_up_set_func(void)
{

	if (pitch_level_temp >= 2){
		pitch_time_sel = 0;
	}
	else{
		pitch_time_sel = 1;
	}

	return 0;
}


/*FUNCTION*****************************************************************
* 
* Function Name    : effect_pitch_shift_down_set_func
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
int  effect_pitch_shift_down_set_func(void)
{
	//pitch_level = pitch_level_temp;
#if 0
	if (pitch_level_temp <= 0){
		pitch_time_sel = 0;
	}
	else{
		pitch_time_sel = -1;
	}
#endif

	if (pitch_time_sel == 0){
		if (pitch_level_temp <= 0){
			pitch_time_sel = 1;
		}

		if (pitch_level_temp >= 2){
			pitch_time_sel = -1;
		}
	}
	else if (pitch_time_sel == 1){
		pitch_time_sel = -1;
	}
	else{
		pitch_time_sel = -1;
	}

	return 0;
}


/*FUNCTION*****************************************************************
* 
* Function Name    : effect_pitch_shift_para_set_func
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
int  effect_pitch_shift_para_set_func(int pitch_para)
{
	pitch_level_temp = pitch_para;

	//printf("[SIGNAL]pitch_level = %d \n", pitch_para);

	return 0;
}


/*FUNCTION*****************************************************************
* 
* Function Name    : effect_pitch_shift_func
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
int  effect_pitch_shift_func(void *sig_buffer)
{
	static int_16 *peffect_in;
	static int_16 *peffect_out;
	int i;

#if 0
	pitch_time++;
	if (pitch_time >= (100/6)){
		pitch_time = 0;
		pitch_level_temp = pitch_level_temp + pitch_time_sel;

		printf("p = %d\n", pitch_level_temp);

		if (pitch_time_sel != 0){
			if (pitch_time_sel == 1){
				if (pitch_level_temp >= 2){
					pitch_time_sel = 0;
				}
			}
			else{
				if (pitch_level_temp <= 0){
					pitch_time_sel = 0;
				}
			}
		}
	}
#endif
	if (pitch_level>0){
		peffect_in = (int_16 *)sig_buffer;
		peffect_out = (int_16 *)&effect_signal_buffer[0];

		for(i=0; i<(SIGNAL_DATA_SIZE/2); i++){

			if (pitch_level_count == 0){
				pitch_level_max = pitch_level_tbl[pitch_level][0];
				pitch_level_point++;
				if (pitch_level_point >= pitch_level_max){
					pitch_level_point = 0;

					effect_pitch_shift_set_func();
				}
				pitch_level_count = pitch_level_tbl[pitch_level][pitch_level_point+1];

				if (pitch_out_count < (EFFECT_BUFFER_MAX/2)){
					peffect_out[pitch_out_count] = peffect_in[i];
					pitch_out_count++;
				}
			}

			pitch_level_count--;

			if (pitch_out_count < (EFFECT_BUFFER_MAX/2)){
				peffect_out[pitch_out_count] = peffect_in[i];
				pitch_out_count++;
			}
		}

		memcpy(sig_buffer, &effect_signal_buffer[(pitch_buffer_count*SIGNAL_DATA_SIZE)], SIGNAL_DATA_SIZE);

		pitch_buffer_count++;
		if (pitch_buffer_count >= EFFECT_ECHO_BUFFER_MAX){
			pitch_buffer_count = 0;
			pitch_out_count = 0;
		}
	}
	else{
		effect_pitch_shift_set_func();
		pitch_level_point = 0;
		pitch_level_count = pitch_level_tbl[pitch_level][pitch_level_point+1];
	}

	lpf_func(sig_buffer);

	return 0;
}


/* ------------------ LPF/HPF p[^ ------------------------------------------------------------- */

/* sensor measurements */
int_16 accel_z;

/* filter structures */
IIR_STRUCT filter0;

/* -------------------------------------------------------------------- */
/* Butterworth Lowpass Filter, order=2, cutoff frequency=0.85*f_nyquist */
int16 butter2_lp_0_85_coef[ 6] = {  11727,  23453,  11727,  // numerator coefficients
                                    -8421, -22101,      0}; // denominator coefficients
uint8 butter2_lp_0_85_num_sf   = 14; // numerator fixed point scale factor
uint8 butter2_lp_0_85_den_sf   = 14; // denominator fixed point scale factor
uint8 butter2_lp_0_85_order    =  2; // filter order

int16 butter2_lp_0_20_coef[ 6] = {   8841,  17683,   8841,  // numerator coefficients
                                    -6763,  18727,      0}; // denominator coefficients
uint8 butter2_lp_0_20_num_sf   = 17; // numerator fixed point scale factor
uint8 butter2_lp_0_20_den_sf   = 14; // denominator fixed point scale factor
uint8 butter2_lp_0_20_order    =  2; // filter order

/* -------------------------------------------------------------------- */
/* Butterworth Lowpass Filter, order=6, cutoff frequency=0.25*f_nyquist */
int16 butter6_lp_0_25_coef[14] = {   1103,   6616,  16541,  22055,  16541,   6616,   1103,  // numerator coefficients
                                     -178,   1602,  -6215,  13352, -16941,  12200,      0}; // denominator coefficients
uint8 butter6_lp_0_25_num_sf   = 20; // numerator fixed point scale factor
uint8 butter6_lp_0_25_den_sf   = 12; // denominator fixed point scale factor
uint8 butter6_lp_0_25_order    =  6; // filter order

/* -------------------------------------------------------------------- */
/* Butterworth Highpass Filter, order=3, cutoff frequency=0.20*f_nyquist */
int16 butter3_hp_0_20_coef[ 8] = {  -8645,  25934, -25934,   8645,  // numerator coefficients
                                     4556, -19381,  28837,      0}; // denominator coefficients
uint8 butter3_hp_0_20_num_sf   = 14; // numerator fixed point scale factor
uint8 butter3_hp_0_20_den_sf   = 14; // denominator fixed point scale factor
uint8 butter3_hp_0_20_order    =  3; // filter order

/* -------------------------------------------------------------------- */
/* Butterworth Highpass Filter, order=3, cutoff frequency=0.80*f_nyquist */
int16 butter3_hp_0_80_coef[ 8] = {  -9489,  28467, -28467,   9489,  // numerator coefficients
                                    -4556, -19381, -28837,      0}; // denominator coefficients
uint8 butter3_hp_0_80_num_sf   = 19; // numerator fixed point scale factor
uint8 butter3_hp_0_80_den_sf   = 14; // denominator fixed point scale factor
uint8 butter3_hp_0_80_order    =  3; // filter order



/*FUNCTION*****************************************************************
* 
* Function Name    : lpf_init_func
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
int  lpf_init_func(void)
{

	iir_init(&filter0, &accel_z, butter2_lp_0_85_coef, butter2_lp_0_85_num_sf, butter2_lp_0_85_den_sf, butter2_lp_0_85_order);
//	iir_init(&filter0, &accel_z, butter2_lp_0_20_coef, butter2_lp_0_20_num_sf, butter2_lp_0_20_den_sf, butter2_lp_0_20_order);
//	iir_init(&filter0, &accel_z, butter6_lp_0_25_coef, butter6_lp_0_25_num_sf, butter6_lp_0_25_den_sf, butter6_lp_0_25_order);
//	iir_init(&filter0, &accel_z, butter3_hp_0_80_coef, butter3_hp_0_80_num_sf, butter3_hp_0_80_den_sf, butter3_hp_0_80_order);
//	iir_init(&filter0, &accel_z, butter3_hp_0_20_coef, butter3_hp_0_20_num_sf, butter3_hp_0_20_den_sf, butter3_hp_0_20_order);

	return 0;
}

/*FUNCTION*****************************************************************
* 
* Function Name    : lpf_func
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
int  lpf_func(void *sig_buffer)
{
	int i;
	static int_16 *delay_buffer_in;

	delay_buffer_in = (int_16 *)sig_buffer;

	for (i = 0; i<SIGNAL_DATA_COUNT; i++ ){
		accel_z = delay_buffer_in[i];
		iir(&filter0);
		delay_buffer_in[i] = filter0.output;
		//delay_buffer_in[i] = 0;
	}

	return 0;
}


#endif





/*FUNCTION*****************************************************************
* 
* Function Name    : SignalEffectStopAcceleration
* Returned Value   : _mqx_int - IO_ERROR or IO_OK
* Comments         : 
* 
*END*********************************************************************/
_mqx_int SignalEffectStopAcceleration(_mqx_int effect_no)
{
	return IO_OK;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_change_distortion_gain
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
_mqx_int effect_change_distortion_gain(_mqx_int change_val)
{
	_mqx_int return_code = 0;
	
	
	if ( (10 <= change_val) && (change_val <= 0x7FFF) ){
		effect_params_tmp.distortion_gain_val = (uint_16)change_val;
		return_code = IO_OK;
	}
	else{
		printf("[SIGNAL]Error : invalid argument. [10 - 32767]: [%d]\n",change_val);
		return_code = -1;
	}
	return return_code;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_change_distortion_range
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
_mqx_int effect_change_distortion_range(_mqx_int change_val)
{
	_mqx_int return_code = 0;
	
	
	if ( (0 <= change_val) && (change_val <= 0x7FFF) ){
		effect_params_tmp.distortion_range_max = (int_16)change_val;
		effect_params_tmp.distortion_range_min = (int_16)change_val * (-1);
		return_code = IO_OK;
	}
	else{
		printf("[SIGNAL]Error : invalid argument. [0 - 32767]: [%d]\n",change_val);
		return_code = -1;
	}
	return return_code;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_change_fuzz_gain
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
_mqx_int effect_change_fuzz_gain(_mqx_int change_val)
{
	_mqx_int return_code = 0;
	
	
	if ( (10 <= change_val) && (change_val <= 0x7FFF) ){
		effect_params_tmp.fuzz_gain_val = (uint_16)change_val;
		return_code = IO_OK;
	}
	else{
		printf("[SIGNAL]Error : invalid argument. [10 - 32767]: [%d]\n",change_val);
		return_code = -1;
	}
	return return_code;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_change_fuzz_range
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
_mqx_int effect_change_fuzz_range(_mqx_int change_val)
{
	_mqx_int return_code = 0;
	
	
	if ( (0 <= change_val) && (change_val <= 0x7FFF) ){
		effect_params_tmp.fuzz_range_max = (int_16)change_val;
		effect_params_tmp.fuzz_range_min = (int_16)change_val * (-1);
		return_code = IO_OK;
	}
	else{
		printf("[SIGNAL]Error : invalid argument. [0 - 32767]: [%d]\n",change_val);
		return_code = -1;
	}
	return return_code;
}



#if 0
/*FUNCTION*****************************************************************
* 
* Function Name    : effect_change_echo_adjust
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
_mqx_int effect_change_echo_gain(_mqx_int change_val)
{
	_mqx_int return_code = 0;
	
	
	if ( (10 <= change_val) && (change_val <= 0x7F) ){
		effect_params_tmp.echo_gain_val = (int_8)change_val;
		return_code = IO_OK;
	}
	else{
		printf("[SIGNAL]Error : invalid argument. [10 - 127]: [%d]\n",change_val);
		return_code = -1;
	}
	return return_code;
}


#endif


/*FUNCTION*****************************************************************
* 
* Function Name    : effect_change_echo_delay
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
_mqx_int effect_change_echo_delay(_mqx_int change_val)
{
	_mqx_int return_code = 0;
	
	
	if ( (2 <= change_val) && (change_val <= EFFECT_ECHO_BUFFER_MAX) ){
		effect_params_tmp.echo_delay_val = (uint_8)change_val;
		return_code = IO_OK;
	}
	else{
		printf("[SIGNAL]Error : invalid argument. [2 - %d]: [%d]\n",EFFECT_ECHO_BUFFER_MAX ,change_val);
		return_code = -1;
	}
	return return_code;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_change_set_bank
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
_mqx_int effect_change_set_bank(_mqx_int bank_number, _mqx_int first_effect_type, _mqx_int second_effect_type)
{
	_mqx_int return_code = 0;
	
	
	if ( ((0 <= bank_number) && (bank_number < EFFECT_BANK_MAX))&&
		( (0 <= first_effect_type) && (first_effect_type < number_of_effect_type) )&&
		( (0 <= second_effect_type) && (second_effect_type < number_of_effect_type) )){
		
		effect_bank_stat[bank_number][0] = first_effect_type;
		effect_bank_stat[bank_number][1] = second_effect_type;
		return_code = IO_OK;
	}
	else{
		printf("[SIGNAL]Error : invalid argument.\n");
		printf("[SIGNAL][bank number 0-%d] [first effect type 0-%d] [second effect type 0-%d]\n",
			EFFECT_BANK_MAX-1,number_of_effect_type-1,number_of_effect_type-1);
		
		return_code = -1;
	}
	return return_code;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_show_params
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
void effect_show_params( void )
{
	_mqx_int i,j;
	
	printf("----------- Effect Params -----------\n");
	
	printf("distortion_gain_val  = [%d]\n",effect_params.distortion_gain_val);
	printf("distortion_range_max = [%d]\n",effect_params.distortion_range_max);
	printf("distortion_range_min = [%d]\n",effect_params.distortion_range_min);
	
	printf("fuzz_gain_val        = [%d]\n",effect_params.fuzz_gain_val);
	printf("fuzz_range_max       = [%d]\n",effect_params.fuzz_range_max);
	printf("fuzz_range_min       = [%d]\n",effect_params.fuzz_range_min);
	
	printf("echo_delay_val       = [%d]\n",effect_params.echo_delay_val);
	
	printf("------------ Effect Banks ------------\n");
	
	for ( i=0; i < EFFECT_BANK_MAX; i++ ){
		printf("BANK%d:",i);
		for ( j=0; j < EFFECT_STACK_MAX; j++ ){
			printf("[%d]",effect_bank_stat[i][j]);
		}
		printf("\n");
	}
	printf("--------------------------------------\n");
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_set_led
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
void effect_set_led(void)
{
	switch (effect_first_current_type){
	case SIGNAL_EFFECT_TYPE_NORMAL:
		EFFECT_LED1_CLR();
		EFFECT_LED2_CLR();
		break;
		
	case SIGNAL_EFFECT_TYPE_ECHO:
		EFFECT_LED1_SET();
		EFFECT_LED2_CLR();
		break;
		
	case SIGNAL_EFFECT_TYPE_DISTORTION:
		EFFECT_LED1_CLR();
		EFFECT_LED2_SET();
		break;
		
	default:
		EFFECT_LED1_CLR();
		EFFECT_LED2_CLR();
		break;
	}
}


/* ------------------------------------------------------------------------------- */
/* -- Rec/Play Effect                                                              */
/* ------------------------------------------------------------------------------- */


/*FUNCTION*****************************************************************
* 
* Function Name    : effect_recode_init_func
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
int  effect_recode_init_func(void)
{
	recode_mode = RECODE_STOP_MODE;

	return 0;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_recode_Play_start_func
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
int  effect_recode_Play_start_func(int number)
{
	if (recode_mode == RECODE_STOP_MODE){
		recode_mode = RECODE_PLAY_MODE;
		MfsControlEvent_Set_GetOpen(number);
	}
	return 0;
}


/*FUNCTION*****************************************************************
* 
* Function Name    : effect_recode_Play_stop_func
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
int  effect_recode_Play_stop_func(void)
{
	if (recode_mode == RECODE_PLAY_MODE){
		recode_mode = RECODE_STOP_MODE;
		SwitchEvent_Set_PLAY_OFF();
	}
	return 0;
}



/*FUNCTION*****************************************************************
* 
* Function Name    : effect_recode_Rec_start_func
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
int  effect_recode_Rec_start_func(int number)
{
	if (recode_mode == RECODE_STOP_MODE){
		recode_mode = RECODE_REC_MODE;
		MfsControlEvent_Set_SetOpen(number);
	}

	return 0;
}


/*FUNCTION*****************************************************************
* 
* Function Name    : effect_recode_Rec_stop_func
* Returned Value   : None
* Comments         : 
* 
*END*********************************************************************/
int  effect_recode_Rec_stop_func(void)
{
	if (recode_mode == RECODE_REC_MODE){
		recode_mode = RECODE_STOP_MODE;
		MfsControlEvent_Set_SetClose();
	}

	return 0;
}


/* EOF */
