/**HEADER********************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved
*
* Copyright (c) 1989-2008 ARC International;
* All Rights Reserved
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
**************************************************************************
*
* $FileName: descriptors.h$
* $Version : 3.0.2.0$
* $Date    : Mar-11-2009$
*
* Comments:
*
*   This file contains USB Descriptor definitions
*
*END************************************************************************/

#ifndef __descriptors_h__
#define __descriptors_h__ 1

/* USB Descriptor Lengths */
#define USB_DESC_LEN_DEV               18
#define USB_DESC_LEN_CFG               9
#define USB_DESC_LEN_STR               2  /* minimum length */
#define USB_DESC_LEN_IF                9
#define USB_DESC_LEN_EP                7
#define USB_DESC_LEN_DEV_QUALIFIER     10
#define USB_DESC_LEN_OTHER_SPEED_CFG   USB_DESC_LEN_CFG

/* USB Descriptor Types */
#define USB_DESC_TYPE_DEV              1
#define USB_DESC_TYPE_CFG              2
#define USB_DESC_TYPE_STR              3
#define USB_DESC_TYPE_IF               4
#define USB_DESC_TYPE_EP               5
#define USB_DESC_TYPE_DEV_QUALIFIER    6
#define USB_DESC_TYPE_OTHER_SPEED_CFG  7
#define USB_DESC_TYPE_IF_POWER         8

/* USB descriptor types - legacy */
#define DEVICE          USB_DESC_TYPE_DEV
#define CONFIGURATION   USB_DESC_TYPE_CFG
#define STRING          USB_DESC_TYPE_STR
#define INTERFACE       USB_DESC_TYPE_IF
#define ENDPOINT        USB_DESC_TYPE_EP

/* USB Standard Device Requests - bmRequestType */
#define USB_DEV_REQ_STD_REQUEST_TYPE_DIR_MASK   0x1
#define USB_DEV_REQ_STD_REQUEST_TYPE_DIR_SHIFT  7
#define USB_DEV_REQ_STD_REQUEST_TYPE_DIR_POS    (USB_DEV_REQ_STD_REQUEST_TYPE_DIR_MASK << \
                                                   USB_DEV_REQ_STD_REQUEST_TYPE_DIR_SHIFT)

#define USB_DEV_REQ_STD_REQUEST_TYPE_TYPE_MASK  0x3
#define USB_DEV_REQ_STD_REQUEST_TYPE_TYPE_SHIFT 5
#define USB_DEV_REQ_STD_REQUEST_TYPE_TYPE_POS   (USB_DEV_REQ_STD_REQUEST_TYPE_TYPE_MASK << \
                                                   USB_DEV_REQ_STD_REQUEST_TYPE_TYPE_SHIFT)

#define USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_MASK  0x1F
#define USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_SHIFT 0
#define USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_POS   (USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_MASK << \
                                                      USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_SHIFT)

#define USB_DEV_REQ_STD_REQUEST_TYPE_DIR_OUT ((0x0 & USB_DEV_REQ_STD_REQUEST_TYPE_DIR_MASK) << \
                                                USB_DEV_REQ_STD_REQUEST_TYPE_DIR_SHIFT)
#define USB_DEV_REQ_STD_REQUEST_TYPE_DIR_IN  ((0x1 & USB_DEV_REQ_STD_REQUEST_TYPE_DIR_MASK) << \
                                                USB_DEV_REQ_STD_REQUEST_TYPE_DIR_SHIFT)

#define USB_DEV_REQ_STD_REQUEST_TYPE_TYPE_STANDARD ((0x0 & USB_DEV_REQ_STD_REQUEST_TYPE_TYPE_MASK) << \
                                                      USB_DEV_REQ_STD_REQUEST_TYPE_TYPE_SHIFT)
#define USB_DEV_REQ_STD_REQUEST_TYPE_TYPE_CLASS    ((0x1 & USB_DEV_REQ_STD_REQUEST_TYPE_TYPE_MASK) << \
                                                      USB_DEV_REQ_STD_REQUEST_TYPE_TYPE_SHIFT)
#define USB_DEV_REQ_STD_REQUEST_TYPE_TYPE_VENDOR   ((0x2 & USB_DEV_REQ_STD_REQUEST_TYPE_TYPE_MASK) << \
                                                      USB_DEV_REQ_STD_REQUEST_TYPE_TYPE_SHIFT)

#define USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_DEVICE    ((0x00 & USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_MASK) << \
                                                            USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_SHIFT)
#define USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_INTERFACE ((0x01 & USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_MASK) << \
                                                            USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_SHIFT)
#define USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_ENDPOINT  ((0x02 & USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_MASK) << \
                                                            USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_SHIFT)
#define USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_OTHER     ((0x03 & USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_MASK) << \
                                                            USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_SHIFT)

/* USB Standard Device Requests - bmRequestType - legacy */
#define IN           USB_DEV_REQ_STD_REQUEST_TYPE_DIR_IN
#define OUT          USB_DEV_REQ_STD_REQUEST_TYPE_DIR_OUT
#define STANDARD     USB_DEV_REQ_STD_REQUEST_TYPE_TYPE_STANDARD
#define CLASS        USB_DEV_REQ_STD_REQUEST_TYPE_TYPE_CLASS
#define VENDOR       USB_DEV_REQ_STD_REQUEST_TYPE_TYPE_VENDOR
#define RQ_DEVICE    USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_DEVICE
#define RQ_INTERFACE USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_INTERFACE
#define RQ_ENDPOINT  USB_DEV_REQ_STD_REQUEST_TYPE_RECIPIENT_ENDPOINT

/* USB Standard Request Codes - bRequest */
#define USB_DEV_REQ_STD_REQUEST_GET_STATUS         0
#define USB_DEV_REQ_STD_REQUEST_CLEAR_FEATURE      1
#define USB_DEV_REQ_STD_REQUEST_SET_FEATURE        3
#define USB_DEV_REQ_STD_REQUEST_SET_ADDRESS        5
#define USB_DEV_REQ_STD_REQUEST_GET_DESCRIPTOR     6
#define USB_DEV_REQ_STD_REQUEST_SET_DESCRIPTOR     7
#define USB_DEV_REQ_STD_REQUEST_GET_CONFIGURATION  8
#define USB_DEV_REQ_STD_REQUEST_SET_CONFIGURATION  9
#define USB_DEV_REQ_STD_REQUEST_GET_INTERFACE      10
#define USB_DEV_REQ_STD_REQUEST_SET_INTERFACE      11
#define USB_DEV_REQ_STD_REQUEST_SYNCH_FRAME        12

/* USB Standard Request Codes - bRequest - legacy */
#define GET_STATUS         USB_DEV_REQ_STD_REQUEST_GET_STATUS
#define CLEAR_FEATURE      USB_DEV_REQ_STD_REQUEST_CLEAR_FEATURE
#define SET_FEATURE        USB_DEV_REQ_STD_REQUEST_SET_FEATURE
#define SET_ADDRESS        USB_DEV_REQ_STD_REQUEST_SET_ADDRESS
#define GET_DESCRIPTOR     USB_DEV_REQ_STD_REQUEST_GET_DESCRIPTOR
#define SET_DESCRIPTOR     USB_DEV_REQ_STD_REQUEST_SET_DESCRIPTOR
#define GET_CONFIGURATION  USB_DEV_REQ_STD_REQUEST_GET_CONFIGURATION
#define SET_CONFIGURATION  USB_DEV_REQ_STD_REQUEST_SET_CONFIGURATION
#define GET_INTERFACE      USB_DEV_REQ_STD_REQUEST_GET_INTERFACE
#define SET_INTERFACE      USB_DEV_REQ_STD_REQUEST_SET_INTERFACE
#define SYNCH_FRAME        USB_DEV_REQ_STD_REQUEST_SYNCH_FRAME

/* Configuration bmAttributes fields */
#define USB_DESC_CFG_ATTRIBUTES_D7_MASK  0x1
#define USB_DESC_CFG_ATTRIBUTES_D7_SHIFT 7
#define USB_DESC_CFG_ATTRIBUTES_D7_POS   (USB_DESC_CFG_ATTRIBUTES_D7_MASK << \
                                          USB_DESC_CFG_ATTRIBUTES_D7_SHIFT)
                                                         
#define USB_DESC_CFG_ATTRIBUTES_SELF_POWERED_MASK  0x1
#define USB_DESC_CFG_ATTRIBUTES_SELF_POWERED_SHIFT 6
#define USB_DESC_CFG_ATTRIBUTES_SELF_POWERED_POS   (USB_DESC_CFG_ATTRIBUTES_SELF_POWERED_MASK << \
                                                      USB_DESC_CFG_ATTRIBUTES_SELF_POWERED_SHIFT)

#define USB_DESC_CFG_ATTRIBUTES_REMOTE_WAKEUP_MASK    0x1
#define USB_DESC_CFG_ATTRIBUTES_REMOTE_WAKEUP_SHIFT   5
#define USB_DESC_CFG_ATTRIBUTES_REMOTE_WAKEUP_POS     (USB_DESC_CFG_ATTRIBUTES_REMOTE_WAKEUP_MASK << \
                                                         USB_DESC_CFG_ATTRIBUTES_REMOTE_WAKEUP_SHIFT)

/* Configuration bmAttributes fields - legacy */
#define BUS_POWERED     (USB_DESC_CFG_ATTRIBUTES_D7_POS)
#define SELF_POWERED    (USB_DESC_CFG_ATTRIBUTES_D7_POS | USB_DESC_CFG_ATTRIBUTES_SELF_POWERED_POS)
#define REMOTE_WAKEUP   (USB_DESC_CFG_ATTRIBUTES_D7_POS | USB_DESC_CFG_ATTRIBUTES_REMOTE_WAKEUP_POS)

/* Endpoint bEndpointAddress fields */
#define USB_DESC_EP_ENDPOINT_ADDRESS_DIR_MASK  0x1
#define USB_DESC_EP_ENDPOINT_ADDRESS_DIR_SHIFT 7
#define USB_DESC_EP_ENDPOINT_ADDRESS_DIR_POS   (USB_DESC_EP_ENDPOINT_ADDRESS_DIR_MASK << \
                                                USB_DESC_EP_ENDPOINT_ADDRESS_DIR_SHIFT)

#define USB_DESC_EP_ENDPOINT_ADDRESS_EP_NUMBER_MASK  0xF
#define USB_DESC_EP_ENDPOINT_ADDRESS_EP_NUMBER_SHIFT 0
#define USB_DESC_EP_ENDPOINT_ADDRESS_EP_NUMBER_POS   (USB_DESC_EP_ENDPOINT_ADDRESS_EP_NUMBER_MASK << \
                                                      USB_DESC_EP_ENDPOINT_ADDRESS_EP_NUMBER_SHIFT)

#define USB_DESC_EP_ENDPOINT_ADDRESS_DIR_OUT ((0x0 & USB_DESC_EP_ENDPOINT_ADDRESS_DIR_MASK) << \
                                                USB_DESC_EP_ENDPOINT_ADDRESS_DIR_SHIFT)
#define USB_DESC_EP_ENDPOINT_ADDRESS_DIR_IN  ((0x1 & USB_DESC_EP_ENDPOINT_ADDRESS_DIR_MASK) << \
                                                USB_DESC_EP_ENDPOINT_ADDRESS_DIR_SHIFT)

/* Endpoint bmAttributes fields */
#define USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_MASK  0x3
#define USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_SHIFT 0
#define USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_POS   (USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_MASK << \
                                                      USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_SHIFT)

#define USB_DESC_EP_ATTRIBUTES_SYNC_TYPE_MASK  0x3
#define USB_DESC_EP_ATTRIBUTES_SYNC_TYPE_SHIFT 2
#define USB_DESC_EP_ATTRIBUTES_SYNC_TYPE_POS   (USB_DESC_EP_ATTRIBUTES_SYNC_TYPE_MASK << \
                                                USB_DESC_EP_ATTRIBUTES_SYNC_TYPE_SHIFT)

#define USB_DESC_EP_ATTRIBUTES_USAGE_TYPE_MASK  0x3
#define USB_DESC_EP_ATTRIBUTES_USAGE_TYPE_SHIFT 4
#define USB_DESC_EP_ATTRIBUTES_USAGE_TYPE_POS   (USB_DESC_EP_ATTRIBUTES_USAGE_TYPE_MASK << \
                                                   USB_DESC_EP_ATTRIBUTES_USAGE_TYPE_SHIFT)

#define USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_CONTROL     ((0x0 & USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_MASK) << \
                                                            USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_SHIFT)
#define USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_ISOCHRONOUS ((0x1 & USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_MASK) << \
                                                            USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_SHIFT)
#define USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_BULK        ((0x2 & USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_MASK) << \
                                                            USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_SHIFT)
#define USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_INTERRUPT   ((0x3 & USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_MASK) << \
                                                            USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_SHIFT)

/* Endpoint bmAttributes fields - legacy */
#define CONTROL      USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_CONTROL
#define ISOCHRONOUS  USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_ISOCHRONOUS
#define BULK         USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_BULK
#define INTERRUPT    USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_INTERRUPT
#define DISABLED     0xFF

/* Standard Feature Selectors */
#define USB_DEV_REQ_STD_FEATURE_SELECTOR_ENDPOINT_HALT         0
#define USB_DEV_REQ_STD_FEATURE_SELECTOR_DEVICE_REMOTE_WAKEUP  1
#define USB_DEV_REQ_STD_FEATURE_SELECTOR_TEST_MODE             2

/* Standard Feature Selectors - legacy */
#define DEVICE_REMOTE_WAKEUP  USB_DEV_REQ_STD_FEATURE_SELECTOR_DEVICE_REMOTE_WAKEUP
#define ENDPOINT_HALT         USB_DEV_REQ_STD_FEATURE_SELECTOR_ENDPOINT_HALT

/* Structure containing fields common to all Standard Descriptors */
typedef struct {
   uint_8 bLength;
   uint_8 bDescriptorType;
} USB_DESC_STRUCT, _PTR_ USB_DESC_STRUCT_PTR;

/* Structure for USB Device Descriptors */
typedef struct {
   USB_DESC_STRUCT desc;
   uint_16  bcdUSB;
   uint_8   bDeviceClass;
   uint_8   bDeviceSubClass;
   uint_8   bDeviceProtocol;
   uint_8   bMaxPacketSize0;
   uint_16  idVendor;
   uint_16  idProduct;
   uint_16  bcdDevice;
   uint_8   iManufacturer;
   uint_8   iProduct;
   uint_8   iSerialNumber;
   uint_8   bNumConfigurations;
} USB_DESC_DEV_STRUCT, _PTR_ USB_DESC_DEV_STRUCT_PTR;

/* Structure for USB Device Descriptors - legacy */
typedef struct {
   uint_8 bLength;
   uint_8 bDescriptorType;
   uint_8 bcdUSBL;
   uint_8 bcdUSBH;
   uint_8 bDeviceClass;
   uint_8 bDeviceSubClass;
   uint_8 bDeviceProtocol;
   uint_8 bMaxPacketSize0;
   uint_8 idVendorL;
   uint_8 idVendorH;
   uint_8 idProductL;
   uint_8 idProductH;
   uint_8 bcdDeviceL;
   uint_8 bcdDeviceH;
   uint_8 iManufacturer;
   uint_8 iProduct;
   uint_8 iSerialNumber;
   uint_8 bNumConfigurations;
} USB_DEVICE_DESC;

/* Structure for USB Device Qualifier Descriptors */
typedef struct {
   USB_DESC_STRUCT desc;
   uint_16  bcdUSB;
   uint_8   bDeviceClass;
   uint_8   bDeviceSubClass;
   uint_8   bDeviceProtocol;
   uint_8   bMaxPacketSize0;
   uint_8   bNumConfigurations;
   uint_8   bReserved;
} USB_DESC_DEV_QUALIFIER_STRUCT, _PTR_ USB_DESC_DEV_QUALIFIER_STRUCT_PTR;

/* Structure for USB Configuration Descriptors */
typedef struct {
   USB_DESC_STRUCT desc;
   uint_16  wTotalLength;
   uint_8   bNumInterfaces;
   uint_8   bConfigurationValue;
   uint_8   iConfiguration;
   uint_8   bmAttributes;
   uint_8   bMaxPower;
} USB_DESC_CFG_STRUCT, _PTR_ USB_DESC_CFG_STRUCT_PTR;

/* Structure for USB Configuration Descriptors - legacy */
typedef struct {
   uint_8 bLength;
   uint_8 bDescriptorType;
   uint_8 wTotalLengthL;
   uint_8 wTotalLengthH;
   uint_8 bNumInterfaces;
   uint_8 bConfigurationValue;
   uint_8 iConfiguration;
   uint_8 bmAttributes;
   uint_8 maxPower;
} USB_CONFIG_DESC;

/* Structure for USB Other Speed Configuration Descriptors */
typedef USB_DESC_CFG_STRUCT         USB_DESC_OTHER_SPEED_CFG_STRUCT;
typedef USB_DESC_CFG_STRUCT _PTR_   USB_DESC_OTHER_SPEED_CFG_STRUCT_PTR;

/* Structure for USB Interface Descriptors */
typedef struct {
   USB_DESC_STRUCT desc;
   uint_8 bInterfaceNumber;
   uint_8 bAlternateSetting;
   uint_8 bNumEndpoints;
   uint_8 bInterfaceClass;
   uint_8 bInterfaceSubClass;
   uint_8 bInterfaceProtocol;
   uint_8 iInterface;
} USB_DESC_IF_STRUCT, _PTR_ USB_DESC_IF_STRUCT_PTR;

/* Structure for USB Interface Descriptors - legacy */
typedef struct {
   uint_8 bLength;
   uint_8 bDescriptorType;
   uint_8 bInterfaceNumber;
   uint_8 bAlternateSetting;
   uint_8 bNumEndpoints;
   uint_8 bInterfaceClass;
   uint_8 bInterfaceSubClass;
   uint_8 bInterfaceProtocol;
   uint_8 iInterface;
} USB_INTERFACE_DESC;

/* Structure for USB Endpoint Descriptors */
typedef struct {
   USB_DESC_STRUCT desc;
   uint_8   bEndpointAddress;
   uint_8   bmAttributes;
   uint_16  wMaxPacketSize;
   uint_8   bInterval;
} USB_DESC_EP_STRUCT, _PTR_ USB_DESC_EP_STRUCT_PTR;

/* Structure for USB Endpoint Descriptors - legacy */
typedef struct {
   uint_8 bLength;
   uint_8 bDescriptorType;
   uint_8 bEndpointAddress;
   uint_8 bmAttributes;
   uint_8 wMaxPacketSizeL;
   uint_8 wMaxPacketSizeH;
   uint_8 bInterval;
} USB_ENDPOINT_DESC;

/* Structure for USB String Zero Descriptors */
typedef struct {
   USB_DESC_STRUCT desc;
   uint_16 * wLANGID_ptr;
} USB_DESC_STR_ZERO_STRUCT, _PTR_ USB_DESC_STR_ZERO_STRUCT_PTR;

/* Structure for USB String Descriptors */
typedef struct {
   USB_DESC_STRUCT desc;
   uint_8 * bString;
} USB_DESC_STR_STRUCT, _PTR_ USB_DESC_STR_STRUCT_PTR;

/* Structure for USB String Descriptors - legacy */
typedef struct {
   uint_8 bLength;
   uint_8 bDescriptorType;
   uint_8 * bString;
} USB_STRING_DESC;

/* Union for accessing all Standard Descriptors */
typedef union {
   USB_DESC_STRUCT                  desc;
   USB_DESC_DEV_STRUCT              dev_desc;
   USB_DESC_DEV_QUALIFIER_STRUCT    dev_qualifier_desc;
   USB_DESC_CFG_STRUCT              cfg_desc;
   USB_DESC_OTHER_SPEED_CFG_STRUCT  other_speed_cfg_desc;
   USB_DESC_IF_STRUCT               if_desc;
   USB_DESC_EP_STRUCT               ep_desc;
   USB_DESC_STR_ZERO_STRUCT         str_zero_desc;
   USB_DESC_STR_STRUCT              str_desc;
} USB_DESC_UNION, _PTR_ USB_DESC_UNION_PTR;

/* Device descriptors are always 18 bytes - legacy */
typedef struct {
   USB_DEVICE_DESC device_desc;
   USB_CONFIG_DESC config_desc;
   USB_INTERFACE_DESC interface_desc;
   USB_ENDPOINT_DESC endpoint_desc;
} DESCRIPTOR_STRUCT;

#endif