/**HEADER********************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
**************************************************************************
*
* $FileName: adc_mcf51xx.c$
* $Version : 3.0.6.0$
* $Date    : Apr-21-2009$
*
* Comments:
*
*   This file contains the nulldisk driver functions
*
*END************************************************************************/

#include <mqx.h>
#include <bsp.h>
#include <fio.h>
#include <fio_prv.h>
#include "io.h"
#include "adc.h"
#include "adc_prv.h"

//#include "adc_mcf51xx.h"

#if BSPCFG_ENABLE_ADC

#include <lwtimer.h>


extern volatile ADC_CHANNEL_PTR adc_ch[];

/* Declarations of IRQ handlers */
static void adc_complete_isr(pointer);

static LWTIMER_PERIOD_STRUCT tm_que;
static _mqx_uint g_time = 0, g_wait = 0, g_last = 0;
static LWTIMER_STRUCT_PTR timer;

//static LWSEM_STRUCT convert_sem;
//static ADC_CHANNEL_GENERIC_PTR measured_ch;
static uint_32 ch_mask = 0, last_mask = 0;

static uint_32 _adc_result[ADC_HW_CHANNELS];




static void _adc_hw_tick(pointer data_ptr) {
    g_time++;
    
    if (g_last + g_wait <= g_time) {
        
    //if (++g_time == wait) {
        g_last = g_time;
        _adc_scheduler(g_time * (1000000 / _time_get_ticks_per_sec()));
    }
}


_mqx_uint _adc_hw_get_time(void) {
    return g_time * (1000000 / _time_get_ticks_per_sec());
}

_mqx_int _adc_hw_run(void) {
    return _lwtimer_add_timer_to_queue(&tm_que, timer, 0, _adc_hw_tick, (pointer)&g_time);
}

_mqx_int _adc_hw_stop(void) {
    return _lwtimer_cancel_timer(timer);
}

    
/*FUNCTION*****************************************************************
* 
* Function Name    : _adc_hw_init
* Returned Value   : _mqx_int
* Comments         :
*    Performs CPU specific initialization for ADC driver
*
*END*********************************************************************/

_mqx_int _adc_hw_init(const ADC_PTR adc) {
    VMCF51CN_STRUCT_PTR mcf51cn_ptr = (VMCF51CN_STRUCT_PTR)BSP_IPSBAR;
    _mqx_uint req_div, div, clk_src;
    void (_CODE_PTR_ old_isr_adc1)(pointer);
    void (_CODE_PTR_ old_isr_timer)(pointer);
    pointer old_isr_adc1_param;
        
    // check adc conf
    if (adc->g.resolution != ADC_RESOLUTION_8BIT &&
        adc->g.resolution != ADC_RESOLUTION_10BIT &&
        adc->g.resolution != ADC_RESOLUTION_12BIT)
        return IO_ERROR; /* cannot set ADC resolution */
        
    if (BSP_BUS_CLOCK / ADC_MAX_CLOCK > 16)
        return IO_ERROR;    // cannot set correct ADCK (from bus clock)
        
/*    if (_lwsem_create(&convert_sem, 0) != MQX_OK)
        return IO_ERROR;*/
    
    if (_lwtimer_create_periodic_queue(&tm_que, 1, 0) != MQX_OK)
        return IO_ERROR;
    
    timer = (LWTIMER_STRUCT_PTR)_mem_alloc(sizeof(LWTIMER_STRUCT));
    if (!timer)
        return IO_ERROR;
        
    // enable adc clock
    mcf51cn_ptr->SIM.SCGC1 |= MCF51XX_SCGC1_ADC_MASK;

    // set adc clock
    clk_src = (BSP_BUS_CLOCK / ADC_MAX_CLOCK > 8) ? 1 : 0;
    
    req_div = (clk_src) ? (BSP_BUS_CLOCK / 2 / ADC_MAX_CLOCK) : (BSP_BUS_CLOCK / ADC_MAX_CLOCK);
    for (div = 0; (req_div > (1 << div)) && (div < 4); div++) {}
    
    // save exact ADC hw clock frequency
    adc->g.adc_frq = (clk_src) ? (BSP_BUS_CLOCK / 2 / ADC_MAX_CLOCK / (1 << div)) : (BSP_BUS_CLOCK / ADC_MAX_CLOCK / (1 << div));
    
    mcf51cn_ptr->ADC.ADCCFG =
        MCF51XX_ADCCFG_ADLPC(0) |      // high power consuption
        MCF51XX_ADCCFG_ADIV(div) |    // divider
        MCF51XX_ADCCFG_ALSMP(0) |      // short sample time
        MCF51XX_ADCCFG_MODE(adc->g.resolution) |   // mode
        MCF51XX_ADCCFG_ADICLK(clk_src);  // clock source - bus clock

    // adc isr
	old_isr_adc1_param = _int_get_isr_data(BSP_ADC_INTERRUPT_VECTOR);
	old_isr_adc1 = _int_install_isr(BSP_ADC_INTERRUPT_VECTOR, adc_complete_isr, NULL);
	
	if (old_isr_adc1 == NULL)
	    return IO_ERROR;

    // adc conf
    mcf51cn_ptr->ADC.ADCSC2 = 0;
    //mcf51cn_ptr->ADC.ADCSC1 = MCF51XX_ADCSC1_AIEN_MASK;

    return IO_OK;
}

/*FUNCTION*****************************************************************
* 
* Function Name    : _adc_hw_deinit
* Returned Value   : _mqx_int
* Comments         :
*    Performs CPU specific deinitialization for ADC driver
*
*END*********************************************************************/

_mqx_int _adc_hw_deinit() {
	return IO_OK;
}

/*FUNCTION*****************************************************************
* 
* Function Name    : _adc_cpu_channel_init
* Returned Value   : int_32
* Comments         :
*    Returns result of ioctl operation.
*
*END*********************************************************************/

_mqx_int _adc_hw_channel_init
   (
       /* number of hw channel */
       _mqx_uint nr
   )
{
    _mqx_int i;

    if (adc_ch[nr]->g.source > ADC_SOURCE_AN7)
        return IO_ERROR; /* ADC_SOURCE_ constant does not match any source signal for this CPU */
      
    /* configure GPIO and MUX for ADC */
    return _bsp_adc_io_init(adc_ch[nr]->g.source);
}

/*FUNCTION*****************************************************************
* 
* Function Name    : _adc_hw_channel_deinit
* Returned Value   : _mqx_int
* Comments         :
*    Performs CPU specific deinitialization for ADC channel
*
*END*********************************************************************/

_mqx_int _adc_hw_channel_deinit(ADC_CHANNEL_PTR channel) {
	return IO_OK;
}

void _adc_hw_measure(ADC_HW_CHANNEL_MASK mask) {
    _mqx_int i;
    ADC_HW_CHANNEL_MASK tmp = mask;
    
    ch_mask = mask;
    last_mask = mask;
    
    for (i = 0; i < ADC_HW_CHANNELS && tmp; i++, tmp >>= 1) {
        if (tmp & 1) {
            ((VMCF51CN_STRUCT_PTR)BSP_IPSBAR)->ADC.ADCSC1 = MCF51XX_ADCSC1_AIEN_MASK | MCF51XX_ADCSC1_ADCH(i);
            break;
        }
    }
        
/*    if (ch_mask) {
        ch_mask |= (1 << channel->g.number);
    }
    else {
        ch_mask |= (1 << channel->g.number);
        
        measured_ch = channel;
    
        ((VMCF51CN_STRUCT_PTR)BSP_IPSBAR)->ADC.ADCSC1 = MCF51XX_ADCSC1_AIEN_MASK | MCF51XX_ADCSC1_ADCH(channel->g.source);
    }
    */
}

/*FUNCTION*****************************************************************
* 
* Function Name    : adc_cpu_isr
* Returned Value   : void
* Comments         : IRQ handler for completed AD conversion(s)
*    In adc_c.channels_to_next_adc we have list of channels that
*    have been sampled and whole we need to handle the results.
*    We will copy results from AD converter to channels buffers.
*    Since AD converter can hold 8 samples a time, we check if any
*    other channel needs to be sampled and if yes, prepare next AD
*    conversion.
*
*END*********************************************************************/

void adc_complete_isr
   (
      /* [IN] the address of the device specific information */
      pointer parameter
   )
{ /* Body */
    VMCF51CN_ADC_STRUCT_PTR adc_ptr = &((VMCF51CN_STRUCT_PTR)BSP_IPSBAR)->ADC;
    //ADC_CHANNEL_PTR channel = *(ADC_CHANNEL_PTR*)parameter;
    
    _mqx_uint ch = adc_ptr->ADCSC1 & MCF51XX_ADCSC1_ADCH_MASK;
    
    _mqx_int i;
    ADC_HW_CHANNEL_MASK mask;
    
    _adc_result[ch] = ((adc_ptr->ADCRH << 8) | adc_ptr->ADCRL);

    ch_mask &= ~(1 << ch);
    
    if (ch_mask) {
        // next conversion...
        
        for (i = ch + 1, mask = (ch_mask >> (ch + 1)); i < ADC_HW_CHANNELS && mask; i++) {
            if (mask & 1) {
                //measured_ch = adc_ch[i];
    
                ((VMCF51CN_STRUCT_PTR)BSP_IPSBAR)->ADC.ADCSC1 = MCF51XX_ADCSC1_AIEN_MASK | MCF51XX_ADCSC1_ADCH(i);
                break;
            }
                
            mask >>= 1;
        }
    }
    else {
        // measure finished
        extern void _adc_conv_end(uint_32 *, ADC_HW_CHANNEL_MASK);
        
        _adc_conv_end(_adc_result, last_mask);
    }
}

void _adc_hw_set_wait(_mqx_uint time) {
    g_wait = (time * _time_get_ticks_per_sec()) / 1000000;
}


/*FUNCTION****************************************************************
* 
* Function Name    : _bsp_get_adc_base_address
* Returned Value   : pointer to base of ADC registers
* Comments         :
*    This function returns base address of ADC related register space.
*
*END*********************************************************************/

pointer _bsp_get_adc_base_address
(
	void
)
{
    return (pointer)(&((VMCF51CN_STRUCT_PTR)BSP_IPSBAR)->ADC);
}

#endif // BSPCFG_ENABLE_ADC
