/**HEADER********************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved
*
* Copyright (c) 2004-2008 Embedded Access Inc.;
* All Rights Reserved
*
* Copyright (c) 1989-2008 ARC International;
* All Rights Reserved
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
**************************************************************************
*
* $FileName: qspi_polled_mcf52xx.c$
* $Version : 3.0.6.0$
* $Date    : Apr-14-2009$
*
* Comments:
*
*   The file contains functions to program queued serial peripheral
*   interface on Freescale Coldfire 52xx CPU
*
*END************************************************************************/
#include "mqx.h"
#include "bsp.h"
#include "io_prv.h"
#include "fio_prv.h"
//#include "timer.h"
#include "../qspi.h"
#include "../qspi_polled_prv.h"
#include "qspi_mcf52xx.h"
#include "qspi_mcf52xx_prv.h"

extern uint_32 _mcf52xx_qspi_polled_init(MCF52XX_QSPI_INIT_STRUCT_PTR, pointer _PTR_, char_ptr);
extern uint_32 _mcf52xx_qspi_polled_deinit(MCF52XX_QSPI_INIT_STRUCT_PTR, MCF52XX_QSPI_INFO_STRUCT_PTR);
extern int_32  _mcf52xx_qspi_polled_ioctl(MCF52XX_QSPI_INFO_STRUCT_PTR, uint_32, _mqx_uint_ptr);
extern int_32  _mcf52xx_qspi_polled_rx(IO_QSPI_POLLED_DEVICE_STRUCT_PTR, uchar_ptr, _mqx_int);
extern int_32  _mcf52xx_qspi_polled_tx(IO_QSPI_POLLED_DEVICE_STRUCT_PTR, uchar_ptr, _mqx_int);

pointer _bsp_get_qspi_base_address(uint_8);
uint_32 _bsp_get_qspi_vector(uint_8);

/*FUNCTION*-------------------------------------------------------------------
*
* Function Name    : _bsp_get_i2c_base_address
* Returned Value   : Address upon success, NULL upon failure
* Comments         :
*    This function returns the base register address of the corresponding I2C
*
*END*----------------------------------------------------------------------*/
pointer _bsp_get_qspi_base_address
(
	uint_8 dev_num
)
{
#if MQX_CPU == 5225
    if (dev_num > 0)
       return NULL;
	return (pointer)&(((VMCF5225_STRUCT_PTR)_PSP_GET_IPSBAR())->QSPI);
#elif MQX_CPU == 5223
    if (dev_num > 0)
       return NULL;
	return (pointer)&(((VMCF5223_STRUCT_PTR)_PSP_GET_IPSBAR())->QSPI);
#elif MQX_CPU == 5222
    if (dev_num > 0)
       return NULL;
	return (pointer)&(((VMCF5222_STRUCT_PTR)_PSP_GET_IPSBAR())->QSPI);
#else
    #error Processor not supported yet in QSPI driver
#endif
}

/*FUNCTION*-------------------------------------------------------------------
*
* Function Name    : _bsp_get_i2c_vector
* Returned Value   : Vector number upon success, 0 upon failure
* Comments         :
*    This function returns desired interrupt vector number for specified I2C
*
*END*----------------------------------------------------------------------*/
uint_32 _bsp_get_qspi_vector
(
	uint_8 dev_num
)
{
#if MQX_CPU == 5225
    if (dev_num > 0)
       return 0;
	return MCF5225_INT_QSPI;
#elif MQX_CPU == 5223
    if (dev_num > 0)
       return 0;
	return MCF5223_INT_QSPI;
#elif MQX_CPU == 5222
    if (dev_num > 0)
       return 0;
	return MCF5222_INT_QSPI;
#endif
}

                       
/*FUNCTION*-------------------------------------------------------------------
* 
* Function Name    : _mcf52xx_qspi_polled_install
* Returned Value   : uint_32 a task error code or MQX_OK
* Comments         :
*    Install an QSPI device.
*
*END*----------------------------------------------------------------------*/
uint_32 _mcf52xx_qspi_polled_install
   (
      /* [IN] A string that identifies the device for fopen */
      char_ptr           identifier,
  
      /* [IN] The I/O init data pointer */
      pointer            init_data_ptr
   )
{ /* Body */

   return _io_qspi_polled_install(identifier,
      (uint_32 (_CODE_PTR_)(pointer, pointer _PTR_, char_ptr))_mcf52xx_qspi_polled_init,
      (uint_32 (_CODE_PTR_)(pointer, pointer))_mcf52xx_qspi_polled_deinit,
      (int_32 (_CODE_PTR_)(pointer, char_ptr, _mqx_int))_mcf52xx_qspi_polled_rx,
      (int_32 (_CODE_PTR_)(pointer, char_ptr, _mqx_int))_mcf52xx_qspi_polled_tx,
      (int_32 (_CODE_PTR_)(pointer, uint_32, _mqx_uint_ptr))_mcf52xx_qspi_polled_ioctl,
      init_data_ptr);

} /* Endbody */

/*FUNCTION****************************************************************
* 
* Function Name    : _mcf52xx_qspi_polled_init  
* Returned Value   : uint_32 a QSPI error code or QSPI_OK
* Comments         :
*    This function initializes the QSPI module 
*
*END*********************************************************************/
uint_32 _mcf52xx_qspi_polled_init
(
   /* [IN] the initialization information for the device being opened */
   MCF52XX_QSPI_INIT_STRUCT_PTR  init_io_dev_ptr,

   /* [OUT] the address to store device specific information */
   pointer _PTR_                 io_info_ptr_ptr,  

   /* [IN] the rest of the name of the device opened */
   char_ptr                      open_name_ptr
)
{
   VMCF52XX_QSPI_STRUCT_PTR      qspi_ptr;
   MCF52XX_QSPI_INFO_STRUCT_PTR	 io_info_ptr;
   uint_32                       div;
   uint_32                       result = QSPI_OK;
   uint_16                       tmp_reg;

   qspi_ptr = _bsp_get_qspi_base_address(0);

   /* QSPI board specific initialization */
//   _bsp_qspi_initialize(0);
   
   /* Initialize internal data */
   io_info_ptr = (MCF52XX_QSPI_INFO_STRUCT _PTR_)
      _mem_alloc_system_zero((uint_32)sizeof(MCF52XX_QSPI_INFO_STRUCT));

#if MQX_CHECK_MEMORY_ALLOCATION_ERRORS
   if (io_info_ptr == NULL) {
      return(MQX_OUT_OF_MEMORY);
   } /* Endif */
#endif

   *io_info_ptr_ptr = io_info_ptr;
   
   io_info_ptr->INIT       = *init_io_dev_ptr;
   io_info_ptr->INT_BITS   = 0;
   io_info_ptr->BUFFER     = NULL;
   io_info_ptr->INTERRUPTS = 0;
   io_info_ptr->RX_PACKETS = 0;
   io_info_ptr->TX_PACKETS = 0;
   io_info_ptr->TX_ABORT   = 0;

   /* Master mode only */
   qspi_ptr->QMR = QSPI_QMR_MSTR;

   /* Set the QSPI clock baud rate divider */
   div = BSP_SYSTEM_CLOCK / (2 * init_io_dev_ptr->BAUD_RATE);

#ifdef MQX_CHECK_ERRORS
   /* Check for the divider value */
   if((div > 255) || (div < 2)) {
   	  _mem_free(io_info_ptr);
      return QSPI_BRG_FAILD;
   }
#endif

   /* Program the value divider */
   qspi_ptr->QMR |= QSPI_QMR_BAUD(div);

   /* Set up QSPI clock polarity and phase */
   switch ( init_io_dev_ptr->TRANSFER_MODE )
   {
   case (QSPI_TRANSFER_MODE_1):
	  /* Inactive state of QSPI_CLK = logic 0 */
      qspi_ptr->QMR &= ~QSPI_QMR_CPOL;
	  /* QSPI_CLK transitions middle of bit timing */
	  qspi_ptr->QMR &= ~QSPI_QMR_CPHA;
      break;
   case (QSPI_TRANSFER_MODE_2):
	  /* Inactive state of QSPI_CLK = logic 1 */
      qspi_ptr->QMR |= QSPI_QMR_CPOL;
	  /* QSPI_CLK transitions middle of bit timing */
	  qspi_ptr->QMR &= ~QSPI_QMR_CPHA;
      break;
   case (QSPI_TRANSFER_MODE_3):
	  /* Inactive state of QSPI_CLK = logic 0 */
      qspi_ptr->QMR &= ~QSPI_QMR_CPOL;
	  /* QSPI_CLK transitions begining of bit timing */
	  qspi_ptr->QMR |= QSPI_QMR_CPHA;
      break;
   case (QSPI_TRANSFER_MODE_4):
	  /* Inactive state of QSPI_CLK = logic 1 */
      qspi_ptr->QMR |= QSPI_QMR_CPOL;
	  /* QSPI_CLK transitions begining of bit timing */
	  qspi_ptr->QMR |= QSPI_QMR_CPHA;
      break;
   default:
      return QSPI_TRANSFER_MODE_INVALID;
   } /* Endswitch */

   /* Data output high impedance disable (default value) */
   qspi_ptr->QMR &= ~QSPI_QMR_DOHIE;

   /* Set delay after transfer to be standard: 17/(fsys/2) */
   qspi_ptr->QDLYR |= QSPI_QDLYR_QCD(0x0004);
   
   /* 
   ** set delay chip select to QSPI clock 
   ** to be standard: 1/2 the QSPI_CLK period 
   */
   qspi_ptr->QDLYR |= QSPI_QDLYR_DTL(0x0004);

   /* Enable write collision, Abort bus errors */
   qspi_ptr->QIR = (QSPI_QIR_WCEFB | QSPI_QIR_ABRTB | QSPI_QIR_ABRTL);

   /* Clear all interrupts */
   tmp_reg = QSPI_QIR_WCEF | QSPI_QIR_ABRT | QSPI_QIR_SPIF;
   qspi_ptr->QIR &= ~(tmp_reg);

   /* Set transfer size to default (8 bits) */
   qspi_ptr->QMR |= QSPI_QMR_BITS(QSPI_QMR_8_BITS);
   
   /* Configure GPIO pins */
   _bsp_qspi_io_init (init_io_dev_ptr->CHIP_SELECT);
   
   return (result);
}

/*FUNCTION****************************************************************
* 
* Function Name    : _mcf52xx_qspi_polled_deinit  
* Returned Value   : uint_32 a QSPI error code or QSPI_OK
* Comments         :
*    This function de-initializes the QSPI module 
*
*END*********************************************************************/
uint_32 _mcf52xx_qspi_polled_deinit
(
   /* [IN] the initialization information for the device being opened */
   MCF52XX_QSPI_INIT_STRUCT_PTR     int_io_dev_ptr,

   /* [IN] the address of the device specific information */
   MCF52XX_QSPI_INFO_STRUCT_PTR     io_info_ptr
)
{
   VMCF52XX_QSPI_STRUCT_PTR     qspi_ptr;
   TIME_STRUCT                  time;
   uint_32                      milliseconds;

   qspi_ptr = _bsp_get_qspi_base_address(0);
    
   /* wait while qspi is busy */
   qspi_ptr->QWR |= QSPI_QWR_HALT;

   /* Disable the QSPI */
   qspi_ptr->QMR |= QSPI_QMR_DOHIE;

   return (QSPI_OK);
}

/*FUNCTION****************************************************************
* 
* Function Name    : _mcf52xx_qspi_polled_ioctl
* Returned Value   : QSPI_OK or a QSPI error code.
* Comments         : 
*    This function performs miscellaneous services for
*    the QSPI I/O device.  
*
*END*********************************************************************/
int_32 _mcf52xx_qspi_polled_ioctl
   (
      /* [IN] the address of the device specific information */
      MCF52XX_QSPI_INFO_STRUCT_PTR     io_info_ptr,

      /* [IN] The command to perform */
      uint_32					   cmd,

      /* [IN] Parameters for the command */
      uint_32_ptr                  param_ptr
   )
{ /* Body */
   VMCF52XX_QSPI_STRUCT_PTR qspi_ptr;
   volatile uint_32         tstatus;
   volatile uint_32         tmp_cs;
   volatile uint_32         state_lev;
   _mqx_int                 i;
   uint_32                  result = QSPI_OK;
  
   qspi_ptr = _bsp_get_qspi_base_address(0);
   
   switch (cmd) {
      case IO_IOCTL_SERIAL_GET_BAUD:
         *param_ptr = (uint_32)(io_info_ptr->INIT.BAUD_RATE);
      break;
      case IO_IOCTL_SERIAL_SET_BAUD:
         /* Disable QSPI module and all QSPI interrupts */
         qspi_ptr->QWR = QSPI_QWR_HALT;
		 qspi_ptr->QIR &= ~(QSPI_QIR_WCEFB | QSPI_QIR_ABRTB | 
		                            QSPI_QIR_WCEFE | QSPI_QIR_SPIFE);
		 /* wait for command execution completed */
		 for(i = 0 ; i < 100; i++){;}
         /* Set the frequency divider */
#ifdef MQX_CHECK_ERRORS
         /* Check for the divider value */
         if((*param_ptr > BSP_SYSTEM_CLOCK / 2 / 2) || (*param_ptr < BSP_SYSTEM_CLOCK / 2 / 255))
            return QSPI_BRG_FAILD;
#endif
         io_info_ptr->INIT.BAUD_RATE = *param_ptr;
         qspi_ptr->QMR &= ~QSPI_QMR_BAUD(0x00FF);         
         qspi_ptr->QMR |= QSPI_QMR_BAUD(BSP_SYSTEM_CLOCK / 2 / *param_ptr);
         /* Enable QSPI module */
         qspi_ptr->QWR &= QSPI_QWR_HALT;
         break;
      case IO_IOCTL_QSPI_DISABLE_DEVICE:
         qspi_ptr->QWR |= QSPI_QWR_HALT;
         break;
      case IO_IOCTL_QSPI_ENABLE_DEVICE:
         qspi_ptr->QWR &= ~QSPI_QWR_HALT;
         break;
	  case IO_IOCTL_QSPI_CHIP_SELECT:
	     tmp_cs = *param_ptr;
		 if	( (tmp_cs == QSPI_CS0) || (tmp_cs == QSPI_CS1) ||
		      (tmp_cs == QSPI_CS2) || (tmp_cs == QSPI_CS3) ) {
		    io_info_ptr->INIT.CHIP_SELECT = *param_ptr;
		    _bsp_qspi_io_init (io_info_ptr->INIT.CHIP_SELECT);
		 }
		 else
		    result = QSPI_CHIPSELECT_INVALID;
		 break;
	  case IO_IOCTL_QSPI_SET_CHIP_SELECT_STATE:
	     state_lev = *param_ptr;
	     if(state_lev == QSPI_CHIP_SELECT_SET_0)
	        qspi_ptr->QWR &= ~(QSPI_QWR_CSIV);
		 if(state_lev == QSPI_CHIP_SELECT_SET_1)
		    qspi_ptr->QWR |= QSPI_QWR_CSIV;
	     break;
   	  case IO_IOCTL_QSPI_SET_TRANSFER_MODE:
         switch ( *param_ptr )
         {
         case (QSPI_TRANSFER_MODE_1):
	        /* Inactive state of QSPI_CLK = logic 0 */
            qspi_ptr->QMR &= ~QSPI_QMR_CPOL;
	        /* QSPI_CLK transitions middle of bit timing */
	        qspi_ptr->QMR &= ~QSPI_QMR_CPHA;
            break;
         case (QSPI_TRANSFER_MODE_2):
	        /* Inactive state of QSPI_CLK = logic 1 */
            qspi_ptr->QMR |= QSPI_QMR_CPOL;
	        /* QSPI_CLK transitions middle of bit timing */
	        qspi_ptr->QMR &= ~QSPI_QMR_CPHA;
            break;
         case (QSPI_TRANSFER_MODE_3):
	        /* Inactive state of QSPI_CLK = logic 0 */
            qspi_ptr->QMR &= ~QSPI_QMR_CPOL;
	        /* QSPI_CLK transitions begining of bit timing */
	        qspi_ptr->QMR |= QSPI_QMR_CPHA;
            break;
         case (QSPI_TRANSFER_MODE_4):
	        /* Inactive state of QSPI_CLK = logic 1 */
            qspi_ptr->QMR |= QSPI_QMR_CPOL;
	        /* QSPI_CLK transitions begining of bit timing */
	        qspi_ptr->QMR |= QSPI_QMR_CPHA;
            break;
         default:
            result = QSPI_TRANSFER_MODE_INVALID;
            break;       
         }
   	     break;
   	  case IO_IOCTL_QSPI_SET_TRANSFER_SIZE:
	     if (*param_ptr == QSPI_TRANSFER_8_BITS)
            /* Set transfer size to default (8 bits) */
            qspi_ptr->QMR |= QSPI_QMR_BITS(QSPI_QMR_8_BITS);
		 if (*param_ptr == QSPI_TRANSFER_16_BITS)
            /* Set transfer size to max. (16 bits) */
            qspi_ptr->QMR &= 0xC3FF;
		 break;
   	  case IO_IOCTL_QSPI_GET_LAST_COMMAND_COMPLETE:
   	     tstatus = 0;
		 tstatus = (qspi_ptr->QWR & 0x00F0);
   	     *param_ptr = tstatus;
   	     break;
      case IO_IOCTL_SERIAL_GET_STATS:
         *param_ptr++ = io_info_ptr->INTERRUPTS;
         *param_ptr++ = io_info_ptr->RX_PACKETS;
         *param_ptr++ = io_info_ptr->TX_PACKETS;
         *param_ptr++ = io_info_ptr->TX_ABORT;
         break;
      case IO_IOCTL_SERIAL_CLEAR_STATS:
         io_info_ptr->INTERRUPTS = 0;
         io_info_ptr->RX_PACKETS = 0;
         io_info_ptr->TX_PACKETS = 0;
         io_info_ptr->TX_ABORT = 0;
         break;
      default:
         break;
   } /* Endswitch */
   
   return(result);

} /* Endbody */

/*FUNCTION****************************************************************
* 
* Function Name    :_mcf52xx_qspi_polled_rx
* Returned Value   : Returns the number of bytes received or QSPI error   
* Comments         : 
*   Reads the data into provided array.
*
*END*********************************************************************/
int_32 _mcf52xx_qspi_polled_rx
   (
      /* [IN] The address of the device specific information */
      IO_QSPI_POLLED_DEVICE_STRUCT_PTR int_io_dev_ptr,

      /* [IN] The array characters are to be read from */
      uchar_ptr                        buffer,
      
      /* [IN] Number of char's to receive */
      int_32                           size
   )
{ /* Body */
   MCF52XX_QSPI_INFO_STRUCT_PTR    io_info_ptr;
   VMCF52XX_QSPI_STRUCT_PTR        qspi_ptr;
   TIME_STRUCT                     time;
   uint_16_ptr                     buf_ptr = NULL;
   _mqx_int                        i, num, length = size;
   uint_32                         milliseconds;

   qspi_ptr = _bsp_get_qspi_base_address(0);

   io_info_ptr  = int_io_dev_ptr->DEV_INFO_PTR;
   
   io_info_ptr->BYTE_COUNT = size;
//   io_info_ptr->RX_INDEX = num;
   io_info_ptr->RX_BUF_LEN = io_info_ptr->TOTAL_SIZE;
   
#ifdef MQX_CHECK_ERRORS
   /* Check the chip select */
   if (io_info_ptr->INIT.CHIP_SELECT != QSPI_CS0)
      if (io_info_ptr->INIT.CHIP_SELECT != QSPI_CS1) 
         if (io_info_ptr->INIT.CHIP_SELECT != QSPI_CS2)
            if (io_info_ptr->INIT.CHIP_SELECT != QSPI_CS3)
               return QSPI_CHIPSELECT_INVALID;
#endif

   while (length > 0)
   {
      qspi_ptr->QAR = QSPI_QAR_ADDR(0x0020);

      if (length < DEFAULT_QSPI_RX_BUFFER_SIZE / 2)
         num = length;
      else
         num = DEFAULT_QSPI_RX_BUFFER_SIZE / 2;
      /* Set up to "size" transfer buffer for CSx */
      for (i = 0; i < num; i++)
         qspi_ptr->QDR = QSPI_QCR_CS(io_info_ptr->INIT.CHIP_SELECT) | QSPI_QCR_BITSE | QSPI_QCR_CONT | QSPI_QCR_DT;
      qspi_ptr->QAR = QSPI_QAR_ADDR(0x0000);
      for (i = 0; i < num; i++)
         qspi_ptr->QDR = 0;
   
      /* Reset start queue pointer */
      qspi_ptr->QWR &= ~QSPI_QWR_NEWQP(0xF);
      /* Reset end queue pointer */
      qspi_ptr->QWR &= ~QSPI_QWR_ENDQP(0xF);
      /* Set end queue pointer */
      qspi_ptr->QWR |= QSPI_QWR_ENDQP(num - 1);
   
   
      /* Enable QSPI to start the transfer */
      qspi_ptr->QDLYR |= QSPI_QDLYR_SPE;
   
      /* Wait for transfer to complete */
      _time_get(&time);
      milliseconds = time.MILLISECONDS;
      while ( ((qspi_ptr->QIR) & QSPI_QIR_SPIF) != QSPI_QIR_SPIF ) 
      {
         _time_get(&time);
         if (time.MILLISECONDS < milliseconds)
            time.MILLISECONDS += 1000;
         if ((time.MILLISECONDS - milliseconds) >= QSPI_TIMEOUT) 
         {
            return QSPI_TIMED_OUT;
         } /* Endif */
      } /* Endwhile */
      
      /* clear completion flag */
      qspi_ptr->QIR |= QSPI_QIR_SPIF;

      length -= num;
   
      qspi_ptr->QAR = QSPI_QAR_ADDR(0x0010);
      /* read received bytes from queue */
      while (num > 0)
      {
         /* Store received data from QSPI_CSx into RX buffer */
	  *buffer++ = (uchar)(qspi_ptr->QDR);
          num--;
      } /* Endwhile */
   }

   io_info_ptr->RX_PACKETS++;

   return size;
} /* Endbody */

/*FUNCTION****************************************************************
* 
* Function Name    : _mcf52xx_qspi_polled_tx
* Returned Value   : return number of byte transmitted or QSPI error code
* Comments         : 
*   Writes the provided data buffer and loops until transmission complete
*
*END*********************************************************************/
int_32 _mcf52xx_qspi_polled_tx
   (
      /* [IN] The address of the device specific information */
      IO_QSPI_POLLED_DEVICE_STRUCT_PTR int_io_dev_ptr,

      /* [IN] The array characters are to be read from */
      uchar_ptr                        buffer,
      
      /* [IN] Number of char's to transmit */
      int_32                           size
   )
{ /* Body */
   
   MCF52XX_QSPI_INFO_STRUCT_PTR    io_info_ptr;
   VMCF52XX_QSPI_STRUCT_PTR        qspi_ptr;
   TIME_STRUCT                     time;
   boolean                         flag = FALSE;
   _mqx_int                        num, i, length = size;
   uint_32                         milliseconds;
   volatile uint_16                buf_ptr = 0;

   qspi_ptr = _bsp_get_qspi_base_address(0);

   io_info_ptr  = int_io_dev_ptr->DEV_INFO_PTR;
   io_info_ptr->TOTAL_SIZE = size;
     
#ifdef MQX_CHECK_ERRORS
   /* Check the chip select */
   if (io_info_ptr->INIT.CHIP_SELECT != QSPI_CS0)
      if (io_info_ptr->INIT.CHIP_SELECT != QSPI_CS1) 
         if (io_info_ptr->INIT.CHIP_SELECT != QSPI_CS2)
            if (io_info_ptr->INIT.CHIP_SELECT != QSPI_CS3)
               return ( QSPI_CHIPSELECT_INVALID );

   /* Check data size */
   if ( size == 0 ) 
      return ( QSPI_TX_NO_DATA );
#endif

   /* Clear halt bit */
   if (qspi_ptr->QWR & QSPI_QWR_HALT)
      qspi_ptr->QWR &= ~QSPI_QWR_HALT;

   /* Clear SPIF bit */
   if (qspi_ptr->QIR & QSPI_QIR_SPIF)
      qspi_ptr->QIR |= (QSPI_QIR_SPIF);

   /* Set queue pointer to the first command RAM entry */
   qspi_ptr->QAR = QSPI_QAR_ADDR(0x0020);
   /* use default delay */
   if (qspi_ptr->QDR & QSPI_QCR_DT)
      qspi_ptr->QDR &= QSPI_QDR_COMMAND(~QSPI_QCR_DT);

   if (qspi_ptr->QDR & QSPI_QCR_DSCK)
      qspi_ptr->QDR &= QSPI_QDR_COMMAND(~QSPI_QCR_DSCK);

   /* 
   ** There is 80 bytes of static RAM dedicated to QSPI for queue processing.
   ** 32bytes for Transmit, 32bytes for Receive and 16bytes for Command.
   ** For 32 bytes data we need to set up only 16 command entry and set transfer 
   ** size to 16 bits. That will allow 32 bytes transfer in one round queue. If 
   ** transfer size set to 8 bits, then only 16 bytes data will be transferred 
   ** in one round queue. There is a "ioctl" call to set the transfer size to 8 
   ** and 16 bits. Wrap around is disabled all the time.
   */
   
   /* Disable wrap around */
   qspi_ptr->QWR &= ~QSPI_QWR_WREN;
      
   /* Reset start queue pointer */
   qspi_ptr->QWR &= ~QSPI_QWR_NEWQP(0xF);

   while (length > 0)
   {
      /* Reset end queue pointer */
      qspi_ptr->QWR &= ~QSPI_QWR_ENDQP(0xF);

      if ( length >= (DEFAULT_QSPI_TX_BUFFER_SIZE / 2) ) {
         /* Set queue end pointer at entry DEFAULT_QSPI_TX_BUFFER_SIZE / 2 */
         qspi_ptr->QWR |= QSPI_QWR_ENDQP( DEFAULT_QSPI_TX_BUFFER_SIZE / 2 - 1 );
         num = DEFAULT_QSPI_TX_BUFFER_SIZE / 2;
         flag = FALSE;
      }
      else {
         /* Set queue end pointer at entry less than 16 */
         /* Begin CR2101 */
         qspi_ptr->QWR |= QSPI_QWR_ENDQP(size - 1);
         num = length;
         /* End CR2101 */
         flag = TRUE;
      }

      /* Set queue pointer to the first command RAM entry */
      qspi_ptr->QAR = QSPI_QAR_ADDR(0x0020);

      /* Set up transfer buffer for CSx */
      for (i = 0; i < num; i++)
         qspi_ptr->QDR = QSPI_QCR_CS(io_info_ptr->INIT.CHIP_SELECT) | QSPI_QCR_BITSE | QSPI_QCR_CONT | QSPI_QCR_DT;
      
      /* Set queue pointer to the first transmit RAM entry */
      qspi_ptr->QAR = QSPI_QAR_ADDR(0x0000);

      /* transfer up to DEFAULT_QSPI_TX_BUFFER_SIZE / 2 bytes data to CSx */
      for (i = 0; i < num; i++)
      {
         buf_ptr = 0;

         /* if we transfer more than 8 bit data, then use the next byte as the upper byte */
         if ( qspi_ptr->QMR & QSPI_QMR_BITS(0x0f) != QSPI_QMR_BITS(0x08) ) {
            buf_ptr = ((*buffer++) << 8);
         }

         buf_ptr |= (uchar) *buffer++;

         qspi_ptr->QDR = (uint_16)(buf_ptr);
      } 
      length -= num;
      
      /* Enable QSPI to start the transfer */
      qspi_ptr->QDLYR |= QSPI_QDLYR_SPE;
        
      /* Wait for transfer to complete */
      _time_get(&time);
      milliseconds = time.MILLISECONDS;
      while ( ((qspi_ptr->QIR) & QSPI_QIR_SPIF) != QSPI_QIR_SPIF ) 
      {
         _time_get(&time);
         if (time.MILLISECONDS < milliseconds)
            time.MILLISECONDS += 1000;
         if ((time.MILLISECONDS - milliseconds) >= QSPI_TIMEOUT) 
         {
            return QSPI_TIMED_OUT;
         } /* Endif */
      } /* Endwhile */
      
      qspi_ptr->QIR |= QSPI_QIR_SPIF;
   
   } /* Endwhile (size > 0) */

   io_info_ptr->TX_PACKETS++;
   
   return size;

} /* Endbody */

/* EOF */