#!/usr/bin/perl

# Perl script to generate source code from static web pages - generates static file system.
#
# DIRECTORY STRUCTURE
#  +-data - contain web sources - (html, css, jpg, gif, ...)
#  |   +-index.html
#  |   +-header.jpg
#  |   +-style.css
#  |
#  +-makefsdata
# 
# Run script from directory containing data directory! Script recursively searches files 
# in data subdirectory. Script generates file tfs_data.c which contains converted data 
# (pages, pictures, ...) for fnet http server.
# 
# ClearCase Perl:
# 	ccperl mktfs.pl <input directory> <output file>
#
# 	Example: ccperl mktfs.pl my_web tfs_data.c
# Perl:
# 	perl mktfs.pl <input directory> <output file>
#
# 	Example: perl mktfs.pl my_web tfs_data.c

if (@ARGV != 2) {
	print "Missing parameters!\n\n";
	print "Usage:\n";
	print "perl mktfs.pl <input directory> <output file>\n";
	print "or\n";
	print "mktfs.exe <input directory> <output file>\n";
	exit();
}

if (-e $ARGV[1] && ! -w $ARGV[1]) {
	print "Overwriting read only file, are you sure? ";
	$response = <STDIN>;
	chomp ($response);
	if ($response !~ /[Yy]/) {
		exit();
	}
	system ("attrib -R $ARGV[1]");
}

print "Converting $ARGV[0] directory to file $ARGV[1] ...\n\n";

open(OUTPUT, "> $ARGV[1]");

chdir($ARGV[0]);

opendir(DIR, ".");
@files =  grep { !/^\./ && !/(CVS|~)/ } readdir(DIR);
closedir(DIR);

foreach $file (@files) {  
   
    if(-d $file && $file !~ /^\./) {
	print "Processing directory $file\n";
	opendir(DIR, $file);
	@newfiles =  grep { !/^\./ && !/(CVS|~)/ } readdir(DIR);
	closedir(DIR);
	printf "Adding files @newfiles\n";
	@files = (@files, map { $_ = "$file/$_" } @newfiles);
	next;
    }
}

print(OUTPUT "#include <tfs.h>\n\n");

foreach $file (@files) {
    if(-f $file) {
	
	print "Adding file $file\n";
	
	open(FILE, $file) || die "Could not open file $file\n";

	binmode(FILE);

	$file =~ s-^-/-;
	$fvar = $file;
	$fvar =~ s-/-_-g;
	$fvar =~ s-\.-_-g;

	print(OUTPUT "static const unsigned char data".$fvar."[] = {\n");
	print(OUTPUT "\t/* $file */\n");

	$j = 0;
	$i = 0;        
	while(read(FILE, $data, 1)) {
            if($j == 0) {
                $j = 1;
            } 
            else {
                print(OUTPUT ", ");
            }
	    if($i == 10) {
		print(OUTPUT "\n");
		$i = 0;
	    }
	    if($i == 0) {
		print(OUTPUT "\t");
	    }
	    printf(OUTPUT "%#.2x", unpack("C", $data));
	    $i++;
	}

#	if($file =~ /\w*\.htm[l]?/i) {
#		print(OUTPUT ", 0};\n\n");
#   	}
#	else {
#		print(OUTPUT "};\n\n");
#	}

	print(OUTPUT " };\n\n");

	close(FILE);
	push(@fvars, $fvar);
	push(@pfiles, $file);
    }
}

print(OUTPUT "const TFS_DIR_ENTRY tfs_data[] = {\n");

for($i = 0; $i < @fvars; $i++) {
    $file = $pfiles[$i];
    $fvar = $fvars[$i];

    if($i == 0) {
        $prevfile = "NULL";
    } else {
        $prevfile = "file" . $fvars[$i - 1];
    }

    print(OUTPUT "\t{ \"$file\", 0, ");

#    if($file =~ /index\.htm[l]?/i) {
#	    print(OUTPUT "TFS_FLAG_INDEX");
#       }
#    else {
#	    print(OUTPUT "0");
#    }


    print(OUTPUT "(unsigned char*)data$fvar, sizeof(data$fvar) },\n");
}

print(OUTPUT "\t{ 0, 0, 0, 0 }\n};\n\n");

close(OUTPUT);

print "\nDone.\n";

