;*HEADER********************************************************************
;* 
;* Copyright (c) 2008 Freescale Semiconductor;
;* All Rights Reserved                       
;*
;* Copyright (c) 1989-2008 ARC International;
;* All Rights Reserved
;*
;*************************************************************************** 
;*
;* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
;* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
;* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
;* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
;* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
;* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
;* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
;* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
;* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
;* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
;* THE POSSIBILITY OF SUCH DAMAGE.
;*
;**************************************************************************
;*
;* $FileName: ipsum.cw$
;* $Version : 3.0.2.0$
;* $Date    : Aug-19-2008$
;*
;* Comments:
;*
;*   This file contains the implementation for a one's
;*   complement checksum.
;*   The function can handle ANY alignment for the data area.
;*   It is optimized for the 68020 and above in order to
;*   perform in the best possible manner.
;*
;*END***********************************************************************

        ; Start CR 2042
        ;.file "ipsum.cw"
        ; End CR 2042
        .text
        .section IPSUM,16,x

;FUNCTION*--------------------------------------------------------------
;
; Function Name    : __mem_sum_ip(a, len, src)
; Returned Value   : one's complement checksum
; Comments         :
;       This function calculates a 16 bit checksum over the specified data.
;
;       Note:  This function returns 0 iff all summands are 0.
;
;END*-------------------------------------------------------------------

        .global __mem_sum_ip

__mem_sum_ip:

        clr.l   d1
        clr.l   d0
        move.l  12(sp),a0       ; a0 is the source address
        move.w  10(sp),d1       ; d1 is the number of bytes to checksum
        move.w  6(sp),d0        ; d0 is the initial sum

        move.l  a1,-(sp)
        move.l  d2,-(sp)
        move.l  d3,-(sp)

;
;   Here is the stack frame for this function:
;
;       +20 src (A0)
;       +16 len (D1)
;       +12 a   (D0)
;       +8  return PC
;       +4  old D2
;   A7   => old D3
;

        cmp.l   #3,d1           ; Do we have 3 or less bytes?
        ble     try_words

;
;   Note:  If the source address isn't word aligned, we could align it,
;   perform a byte-swapped sum, and byte-swap the result (this works).
;   However, since RTCS never uses non word aligned addresses, this
;   isn't implemented.
;

        move.l  a0,d2           ; Try to get onto even word
        btst.l  #1,d2
        beq.b   try_longs       ; YES, address is long aligned already
        clr.l   d3              ; NO, align that puppy
        move.w  (a0)+,d3
        add.l   d3,d0
        subq.l  #2,d1

try_longs:
        move.l  d1,d2
        lsr.l   #6,d2           ; 64 bytes summed per loop
        move.l  d2,a1

        move.l  d1,d3           ; 1 long processed by 4 bytes
        and.l   #0x3C,d3
        neg.l   d3
        move.w  #0,ccr          ; Clear X
        jmp     66(pc,d3.l)

do_longs:
        move.l  (a0)+,d3
        addx.l  d3,d0
        move.l  (a0)+,d3
        addx.l  d3,d0
        move.l  (a0)+,d3
        addx.l  d3,d0
        move.l  (a0)+,d3
        addx.l  d3,d0
        move.l  (a0)+,d3
        addx.l  d3,d0
        move.l  (a0)+,d3
        addx.l  d3,d0
        move.l  (a0)+,d3
        addx.l  d3,d0
        move.l  (a0)+,d3
        addx.l  d3,d0

        move.l  (a0)+,d3
        addx.l  d3,d0
        move.l  (a0)+,d3
        addx.l  d3,d0
        move.l  (a0)+,d3
        addx.l  d3,d0
        move.l  (a0)+,d3
        addx.l  d3,d0
        move.l  (a0)+,d3
        addx.l  d3,d0
        move.l  (a0)+,d3
        addx.l  d3,d0
        move.l  (a0)+,d3
        addx.l  d3,d0
        move.l  (a0)+,d3
        addx.l  d3,d0

do_longs_end:
        suba.l  #1,a1
        tst.l   a1
        bge     do_longs

do_word:
        btst    #1,d1           ; Sum last word
        beq     do_byte
        clr.l   d3
        move.w  (a0)+,d3
        addx.l  d3,d0

do_byte:
        btst    #0,d1           ; Sum last byte
        beq.b   done
        clr.l   d3
        addx.l  d3,d0            ; add the carry if there is one.
        move.b  (a0)+,d3
        asl.l   #8,d3
        add.l   d3,d0

done:   move.l  d0,d1           ; Fold 32 bit sum into 16
        swap    d1
        and.l   #0xFFFF,d0
        and.l   #0xFFFF,d1
        addx.l  d1,d0
        move.l  d0,d1
        swap    d1
        add.l   d1,d0
        and.l   #0xFFFF,d0

        move.l  (sp)+,d3
        move.l  (sp)+,d2
        move.l  (sp)+,a1
        rts
        .word    0x0000

try_words:
        move.w  #0,ccr          ; Clear X
        bra.b   do_word

 
; EOF
