/**HEADER********************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved                       
*
* Copyright (c) 1989-2008 ARC International;
* All Rights Reserved
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
**************************************************************************
*
* $FileName: mem_zero.c$
* $Version : 3.0.2.0$
* $Date    : Aug-19-2008$
*
* Comments:
*
*   This file contains the functions for zeroing memory
*
*END************************************************************************/

#include "mqx_inc.h"


/*FUNCTION*-------------------------------------------------------------------
* 
* Function Name    : _mem_zero
* Returned Value   : none
* Comments         :
*   This function zeros the specified number of bytes at the specified
* location.
*   The zeroing is optimized to avoid alignment problems, and attempts
* to zero 32bit numbers optimally
*
*END*----------------------------------------------------------------------*/


void _mem_zero
   (
      /* [IN] the address to start zeroing memory from */
      register pointer loc,

      /* [IN] the number of bytes to zero */
      register uint_32 number_of_bytes
   )
{ /* Body */
#ifdef MQX_USE_SMALL_MEM_ZERO
   if (number_of_bytes) {
      while (number_of_bytes--) {
         *((uchar_ptr)loc)++ = 0;
      } /* Endwhile */  	
   } /* Endif */
#else
   register uint_32 loops;

   if (number_of_bytes > 3) {

      /* Align source on word */
      if ((uint_32)loc & 1) {
         *((uchar_ptr)loc)++ = 0;
         --number_of_bytes;
      } /* Endif */

      /* Align source on longword */
      if ((uint_32)loc & 2) {
         *((uint_16_ptr)loc)++ = 0;
         number_of_bytes -= 2;
      } /* Endif */

      /* Get number of long words
      ** we will unravel the zero loop to zero 64 bytes per loop
      */
      loops = (number_of_bytes >> 6) + 1;  
      switch ((number_of_bytes >> 2) & 0xF) {
         while (--loops) {  /* --loops since avoid while first time in */
                     *((uint_32_ptr)loc)++ = 0;
            case 15: *((uint_32_ptr)loc)++ = 0;
            case 14: *((uint_32_ptr)loc)++ = 0;
            case 13: *((uint_32_ptr)loc)++ = 0;
            case 12: *((uint_32_ptr)loc)++ = 0;
            case 11: *((uint_32_ptr)loc)++ = 0;
            case 10: *((uint_32_ptr)loc)++ = 0;
            case  9: *((uint_32_ptr)loc)++ = 0;
            case  8: *((uint_32_ptr)loc)++ = 0;
            case  7: *((uint_32_ptr)loc)++ = 0;
            case  6: *((uint_32_ptr)loc)++ = 0;
            case  5: *((uint_32_ptr)loc)++ = 0;
            case  4: *((uint_32_ptr)loc)++ = 0;
            case  3: *((uint_32_ptr)loc)++ = 0;
            case  2: *((uint_32_ptr)loc)++ = 0;
            case  1: *((uint_32_ptr)loc)++ = 0;
            case  0: ;
         } /* Endwhile */
      } /* Endswitch */

   } /* Endif */

   /* Handle leftover bytes */
   if ( number_of_bytes & 2 ) {
      *((uchar_ptr)loc)++ = 0;
      *((uchar_ptr)loc)++ = 0;
   }/* Endif */
   if ( number_of_bytes & 1 ) {
      *((uchar_ptr)loc) = 0;
   }/* Endif */
#endif
} /* Endbody */

/* EOF */